@@ -1,6 +1,6 @@
 #' @title Find statistic for model
-#' @description Returns the statistic for a regression model (\emph{t}-statistic,
-#'   \emph{z}-statistic, etc.).
+#' @description Returns the statistic for a regression model (*t*-statistic,
+#'   *z*-statistic, etc.).
 #' @name find_statistic
 #'
 #' @description Small helper that checks if a model is a regression model
@@ -10,7 +10,7 @@
 #' @param ... Currently not used.
 #'
 #' @return A character describing the type of statistic. If there is no
-#'   statistic available with a distribution, \code{NULL} will be returned.
+#'   statistic available with a distribution, `NULL` will be returned.
 #'
 #' @examples
 #' # regression model object
@@ -43,6 +43,10 @@ find_statistic <- function(x, ...) {
     x <- x$analyses[[1]]
   }
 
+  if (inherits(x, "model_fit")) {
+    x <- x$fit
+  }
+
   if (inherits(x, "merModList")) {
     x <- x[[1]]
   }
@@ -55,191 +59,79 @@ find_statistic <- function(x, ...) {
 
   # t-value objects ----------------------------------------------------------
 
-  t.mods <-
-    c(
-      "bayesx",
-      "BBreg",
-      "BBmm",
-      "bcplm",
-      "biglm",
-      "blmerMod",
-      "cch",
-      "censReg",
-      "complmrob",
-      "cpglm",
-      "cpglmm",
-      "crq",
-      "crqs",
-      "drc",
-      "emmGrid",
-      "emm_list",
-      "feis",
-      "felm",
-      "gamlss",
-      "garch",
-      "glmmPQL",
-      "gls",
-      "gmm",
-      "HLfit",
-      "ivreg",
-      "ivFixed",
-      "iv_robust",
-      "ivprobit",
-      "lm",
-      "lm_robust",
-      "lm.beta",
-      "lme",
-      "lmerMod",
-      "lmerModLmerTest",
-      "lmodel2",
-      "lmRob",
-      "lmrob",
-      "lqm",
-      "lqmm",
-      "maxLik",
-      "mixed",
-      "mlm",
-      "multinom",
-      "nlmerMod",
-      "nlrq",
-      "nls",
-      "ols",
-      "orcutt",
-      "pb1",
-      "pb2",
-      "polr",
-      "rlm",
-      "rms",
-      "rlmerMod",
-      "rq",
-      "rqs",
-      "rqss",
-      "speedlm",
-      "spml",
-      "summary.lm",
-      "svyglm",
-      "svyolr",
-      "truncreg",
-      "wbm",
-      "wblm",
-      "yuen"
-    )
+  t.mods <- c(
+    "bayesx", "BBreg", "BBmm", "bcplm", "biglm", "bfsl", "blmerMod",
+    "cch", "censReg", "complmrob", "cpglm", "cpglmm", "crq", "crqs",
+    "drc",
+    "elm",
+    "feis", "felm",
+    "gamlss", "garch", "glmmPQL", "gls", "gmm", "gnls",
+    "HLfit",
+    "ivreg", "ivFixed", "iv_robust", "ivprobit",
+    "lm", "lm_robust", "lm.beta", "lme", "lmerMod", "lmerModLmerTest",
+    "lmodel2", "lmRob", "lmrob", "lqm", "lqmm",
+    "maxLik", "mixed", "mhurdle", "mlm", "multinom",
+    "nlmerMod", "nlrq", "nls",
+    "ols", "orcutt",
+    "pb1", "pb2", "polr",
+    "rlm", "rms", "rlmerMod", "rq", "rqs", "rqss",
+    "selection", "speedlm", "spml", "summary.lm", "svyglm", "svyolr", "systemfit",
+    "truncreg",
+    "varest",
+    "wbm", "wblm",
+    "yuen"
+  )
 
   # z-value objects ----------------------------------------------------------
 
-  z.mods <-
-    c(
-      "aareg",
-      "Arima",
-      "averaging",
-      "betamfx",
-      "betaor",
-      "betareg",
-      "bife",
-      "bglmerMod",
-      "boot_test_mediation",
-      "bracl",
-      "brglm",
-      "brglmFit",
-      "brmultinom",
-      "cglm",
-      "cph",
-      "clm",
-      "clm2",
-      "clmm",
-      "clmm2",
-      "clogit",
-      "coxme",
-      "coxph",
-      "coxr",
-      "crch",
-      "DirichletRegModel",
-      "ergm",
-      "feglm",
-      "flexsurvreg",
-      "gee",
-      "glimML",
-      "glmm",
-      "glmmadmb",
-      "glmmLasso",
-      "glmmTMB",
-      "glmx",
-      "gmnl",
-      "hurdle",
-      "lavaan",
-      "loggammacenslmrob",
-      "logitmfx",
-      "logitor",
-      "LORgee",
-      "lrm",
-      "margins",
-      "metaplus",
-      "mixor",
-      "MixMod",
-      "mjoint",
-      "mle",
-      "mle2",
-      "mlogit",
-      "mclogit",
-      "mmclogit",
-      "mvmeta",
-      "negbin",
-      "negbinmfx",
-      "negbinirr",
-      "nlreg",
-      "objectiveML",
-      "orm",
-      "poissonmfx",
-      "poissonirr",
-      "psm",
-      "probitmfx",
-      "riskRegression",
-      "robmixglm",
-      "rma",
-      "rma.mv",
-      "rma.uni",
-      "rrvglm",
-      "sarlm",
-      "sem",
-      "slm",
-      "survreg",
-      "test_mediation",
-      "tobit",
-      "vglm",
-      "wbgee",
-      "zcpglm",
-      "zeroinfl",
-      "zerotrunc"
-    )
+  z.mods <- c(
+    "aareg", "Arima", "averaging",
+    "betamfx", "betaor", "betareg", "bife", "bifeAPEs", "bglmerMod",
+    "boot_test_mediation", "bracl", "brglm", "brglmFit", "brmultinom", "btergm",
+    "cglm", "cph", "clm", "clm2", "clmm", "clmm2", "clogit", "coxme", "coxph",
+    "coxr", "crch", "crr",
+    "DirichletRegModel",
+    "ergm",
+    "feglm", "flexsurvreg",
+    "gee", "glimML", "glmm", "glmmadmb", "glmmFit", "glmmLasso", "glmmTMB",
+    "glmx", "gmnl",
+    "hurdle",
+    "lavaan", "loggammacenslmrob", "logitmfx", "logitor", "LORgee", "lrm",
+    "margins", "metaplus", "mixor", "MixMod", "mjoint", "mle", "mle2", "mlogit",
+    "mclogit", "mmclogit", "mvmeta", "mvord",
+    "negbin", "negbinmfx", "negbinirr", "nlreg",
+    "objectiveML", "orm",
+    "poissonmfx", "poissonirr", "psm", "probitmfx", "pgmm",
+    "qr", "QRNLMM", "QRLMM",
+    "Rchoice", "riskRegression", "robmixglm", "rma", "rma.mv", "rma.uni", "rrvglm",
+    "Sarlm", "sem", "SemiParBIV", "slm", "survreg", "svy_vglm",
+    "test_mediation", "tobit",
+    "vglm",
+    "wbgee",
+    "zcpglm", "zeroinfl", "zerotrunc"
+  )
 
   # F-value objects ----------------------------------------------------------
 
-  f.mods <-
-    c(
-      "afex_aov",
-      "Anova.mlm",
-      "aov",
-      "aovlist",
-      "anova",
-      "Gam",
-      "manova",
-      "maov",
-      "t1way"
-    )
+  f.mods <- c(
+    "afex_aov", "Anova.mlm", "aov", "aovlist", "anova",
+    "Gam",
+    "manova", "maov",
+    "t1way"
+  )
 
   # chi-squared value objects ------------------------------------------------
 
-  chi.mods <-
-    c(
-      "coxph.penal",
-      "geeglm",
-      "logistf",
-      "MANOVA",
-      "mlma",
-      "nparLD",
-      "RM",
-      "vgam"
-    )
+  chi.mods <- c(
+    "coxph.penal",
+    "epi.2by2",
+    "geeglm",
+    "logistf",
+    "MANOVA", "mlma",
+    "nparLD",
+    "RM",
+    "vgam"
+  )
 
   # mixed bag ----------------------------------------------------------------
 
@@ -247,18 +139,11 @@ find_statistic <- function(x, ...) {
   # which statistic to use will be decided based on the family used
   g.mods <-
     c(
-      "bam",
-      "bigglm",
-      "cgam",
-      "cgamm",
+      "bam", "bigglm",
+      "cgam", "cgamm",
+      "eglm", "emmGrid", "emm_list",
       "fixest",
-      "gam",
-      "glm",
-      "Glm",
-      "glmc",
-      "glmerMod",
-      "glmRob",
-      "glmrob",
+      "gam", "glm", "Glm", "glmc", "glmerMod", "glmRob", "glmrob",
       "pseudoglm",
       "scam",
       "speedglm"
@@ -267,11 +152,8 @@ find_statistic <- function(x, ...) {
   # t-statistic (otherwise z-statistic: "binomial", "poisson")
   g.t.mods <-
     c(
-      "quasi",
-      "gaussian",
-      "quasibinomial",
-      "quasipoisson",
-      "Gamma",
+      "gaussian", "Gamma",
+      "quasi", "quasibinomial", "quasipoisson",
       "inverse.gaussian"
     )
 
@@ -297,33 +179,28 @@ find_statistic <- function(x, ...) {
 
   # no statistic -------------------------------------------------------------
 
-  unsupported.mods <-
-    c(
-      "bcplm",
-      "BFBayesFactor",
-      "brmsfit",
-      "gbm",
-      "list",
-      "MCMCglmm",
-      "mediate",
-      "mlergm",
-      "pairwise.htest",
-      "ridgelm",
-      "splmm",
-      "stanreg",
-      "stanmvreg",
-      "survfit"
-    )
+  unsupported.mods <- c(
+    "bcplm", "BFBayesFactor", "brmsfit",
+    "gbm", "glmmEP",
+    "joint",
+    "list",
+    "MCMCglmm", "mediate", "mlergm",
+    "pairwise.htest",
+    "ridgelm",
+    "splmm", "stanreg", "stanmvreg", "survfit"
+  )
 
   # edge cases ---------------------------------------------------------------
 
+  m_info <- model_info(x, verbose = FALSE, return_family_only = TRUE)
+
   # tweedie-check needs to come first, because glm can also have tweedie
   # family, so this exception needs to be caught before checking for g.mods
 
   tryCatch(
     {
       suppressWarnings(
-        if (!is_multivariate(x) && model_info(x)$is_tweedie) {
+        if (!is_multivariate(x) && .is_tweedie(x, m_info)) {
           return("t-statistic")
         }
       )
@@ -356,7 +233,22 @@ find_statistic <- function(x, ...) {
   }
 
   if (model_class %in% g.mods) {
-    if (model_info(x)$family %in% g.t.mods) {
+    if (model_class %in% c("emmGrid", "emm_list")) {
+      stat <- tryCatch(
+        {
+          df <- get_df(x)
+          if (all(is.na(df)) || all(is.infinite(df))) {
+            "z-statistic"
+          } else {
+            "t-statistic"
+          }
+        },
+        error = function(e) {
+          "t-statistic"
+        }
+      )
+      return(stat)
+    } else if (m_info$family %in% g.t.mods) {
       return("t-statistic")
     } else {
       return("z-statistic")
@@ -367,44 +259,64 @@ find_statistic <- function(x, ...) {
 
   if (model_class %in% unclear.mods) {
     col_names <- colnames(as.data.frame(summary(x)$coefficients))
-    t_names <-
-      c(
-        "t",
-        "t-value",
-        "t value",
-        "t.value",
-        "Pr(>|t|)"
-      )
-    z_names <-
-      c(
-        "z",
-        "z-value",
-        "z value",
-        "z.value",
-        "Pr(>|z|)",
-        "Pr(>|Z|)",
-        "Naive z",
-        "Robust z",
-        "san.z",
-        "Wald Z"
-      )
+
+    t_names <- c(
+      "t",
+      "t-value",
+      "t value",
+      "t.value",
+      "Pr(>|t|)"
+    )
+
+    z_names <- c(
+      "z",
+      "z-value",
+      "z value",
+      "z.value",
+      "Pr(>|z|)",
+      "Pr(>|Z|)",
+      "Naive z",
+      "Robust z",
+      "san.z",
+      "Wald Z"
+    )
+
     f_names <- c("F", "F-value", "F value", "F.value")
+
     chi_names <- c("Chisq", "chi-sq", "chi.sq", "Wald", "W", "Pr(>|W|)")
 
     if (length(col_names) == 0L) {
       return(NULL)
     }
+
     if (any(t_names %in% col_names)) {
       return("t-statistic")
     }
+
     if (any(z_names %in% col_names)) {
       return("z-statistic")
     }
+
     if (any(f_names %in% col_names)) {
       return("F-statistic")
     }
+
     if (any(chi_names %in% col_names)) {
       return("chi-squared statistic")
     }
   }
 }
+
+
+
+
+
+# helper ---------------
+
+.is_tweedie <- function(model, info) {
+  if (info$family %in% c("Student's-t", "t Family", "gaussian", "Gaussian") || grepl("(\\st)$", info$family)) {
+    linear_model <- TRUE
+  }
+  tweedie_fam <- grepl("^(tweedie|Tweedie)", info$family) | grepl("^(tweedie|Tweedie)", info$link_function)
+  (linear_model && tweedie_fam) || inherits(model, c("bcplm", "cpglm", "cpglmm", "zcpglm"))
+}
