@@ -13,19 +13,20 @@
 #'
 #' @return The data that was used to fit the model.
 #'
-#' @note Unlike \code{model.frame()}, which may contain transformed variables
-#'   (e.g. if \code{poly()} or \code{scale()} was used inside the formula to
-#'   specify the model), \code{get_data()} aims at returning the "original",
+#' @note Unlike `model.frame()`, which may contain transformed variables
+#'   (e.g. if `poly()` or `scale()` was used inside the formula to
+#'   specify the model), `get_data()` aims at returning the "original",
 #'   untransformed data (if possible). Consequently, column names are changed
-#'   accordingly, i.e. \code{"log(x)"} will become \code{"x"} etc. for all data
+#'   accordingly, i.e. `"log(x)"` will become `"x"` etc. for all data
 #'   columns with transformed values.
 #'
 #' @examples
-#' data(cbpp, package = "lme4")
-#' cbpp$trials <- cbpp$size - cbpp$incidence
-#' m <- glm(cbind(incidence, trials) ~ period, data = cbpp, family = binomial)
-#' head(get_data(m))
-#' @importFrom stats model.frame na.omit
+#' if (require("lme4")) {
+#'   data(cbpp, package = "lme4")
+#'   cbpp$trials <- cbpp$size - cbpp$incidence
+#'   m <- glm(cbind(incidence, trials) ~ period, data = cbpp, family = binomial)
+#'   head(get_data(m))
+#' }
 #' @export
 get_data <- function(x, ...) {
   UseMethod("get_data")
@@ -58,7 +59,9 @@ get_data.default <- function(x, verbose = TRUE, ...) {
   if (is.null(mf)) {
     mf <- tryCatch(
       {
-        .get_data_from_env(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
+        dat <- .recover_data_from_environment(x)
+        vars <- find_variables(x, flatten = TRUE, verbose = FALSE)
+        dat[, intersect(vars, colnames(dat)), drop = FALSE]
       },
       error = function(x) {
         NULL
@@ -77,19 +80,24 @@ get_data.data.frame <- function(x, ...) {
 
 
 #' @export
-get_data.summary.lm <- function(x, ...) {
+get_data.summary.lm <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
-      .get_data_from_env(x)[, all.vars(x$terms), drop = FALSE]
+      .recover_data_from_environment(x)[, all.vars(x$terms), drop = FALSE]
     },
     error = function(x) {
       NULL
     }
   )
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
+#' @export
+get_data.model_fit <- function(x, verbose = TRUE, ...) {
+  get_data(x$fit, verbose = verbose, ...)
+}
+
 
 
 
@@ -97,19 +105,45 @@ get_data.summary.lm <- function(x, ...) {
 # classical and survival models -----------------------------------------------
 
 
+#' @export
+get_data.mjoint <- function(x, verbose = TRUE, ...) {
+  mf <- tryCatch(
+    {
+      dat <- x$data[[1]]
+      data_columns <- intersect(
+        colnames(dat),
+        unique(c(
+          find_response(x, combine = FALSE, component = "all"),
+          find_variables(x, flatten = TRUE, verbose = FALSE)
+        ))
+      )
+      dat[, data_columns, drop = FALSE]
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
+}
+
+
 #' @rdname get_data
 #' @export
-get_data.gee <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.gee <- function(x,
+                         effects = c("all", "fixed", "random"),
+                         verbose = TRUE,
+                         ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)
-      switch(
-        effects,
-        all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
-        fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
-        random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
+      dat <- .recover_data_from_environment(x)
+      vars <- switch(effects,
+        all = find_variables(x, flatten = TRUE, verbose = FALSE),
+        fixed = find_variables(x, effects = "fixed", flatten = TRUE, verbose = FALSE),
+        random = find_random(x, flatten = TRUE)
       )
+      dat[, intersect(vars, colnames(dat)), drop = FALSE]
     },
     error = function(x) {
       NULL
@@ -123,12 +157,23 @@ get_data.gee <- function(x, effects = c("all", "fixed", "random"), verbose = TRU
 
 #' @rdname get_data
 #' @export
-get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms"), verbose = TRUE, ...) {
+get_data.rqss <- function(x,
+                          component = c("all", "conditional", "smooth_terms"),
+                          verbose = TRUE,
+                          ...) {
   component <- match.arg(component)
+
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)
-      dat[, find_variables(x, effects = "all", component = component, flatten = TRUE), drop = FALSE]
+      dat <- .recover_data_from_environment(x)
+      vars <- find_variables(
+        x,
+        effects = "all",
+        component = component,
+        flatten = TRUE,
+        verbose = FALSE
+      )
+      dat[, intersect(vars, colnames(dat)), drop = FALSE]
     },
     error = function(x) {
       NULL
@@ -140,14 +185,15 @@ get_data.rqss <- function(x, component = c("all", "conditional", "smooth_terms")
 
 
 
-
-
 #' @export
 get_data.gls <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)
-      data_columns <- intersect(colnames(dat), find_variables(x, flatten = TRUE))
+      dat <- .recover_data_from_environment(x)
+      data_columns <- intersect(
+        colnames(dat),
+        find_variables(x, flatten = TRUE, verbose = FALSE)
+      )
       dat[, data_columns, drop = FALSE]
     },
     error = function(x) {
@@ -173,6 +219,17 @@ get_data.nlrq <- get_data.gls
 #' @export
 get_data.robmixglm <- get_data.gls
 
+#' @export
+get_data.selection <- get_data.gls
+
+# if ("lm" %in% names(x)) {
+#   suppressWarnings(get_data(x$lm, verbose = verbose))
+# } else if (!is.null(x$twoStep$lm)) {
+#   suppressWarnings(get_data(x$twoStep$lm, verbose = verbose))
+# } else {
+#   NULL
+# }
+
 
 #' @export
 get_data.lqmm <- function(x, verbose = TRUE, ...) {
@@ -188,13 +245,11 @@ get_data.lqmm <- function(x, verbose = TRUE, ...) {
   .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
+#' @export
+get_data.nls <- get_data.gls
 
-
-
-
-
-
-
+#' @export
+get_data.gnls <- get_data.gls
 
 
 # zero-inflated models -------------------------------------------------------
@@ -202,7 +257,10 @@ get_data.lqmm <- function(x, verbose = TRUE, ...) {
 
 #' @rdname get_data
 #' @export
-get_data.hurdle <- function(x, component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
+get_data.hurdle <- function(x,
+                            component = c("all", "conditional", "zi", "zero_inflated", "dispersion"),
+                            verbose = TRUE,
+                            ...) {
   component <- match.arg(component)
   .return_zeroinf_data(x, component, verbose = verbose)
 }
@@ -216,7 +274,10 @@ get_data.zerotrunc <- get_data.hurdle
 
 #' @rdname get_data
 #' @export
-get_data.zcpglm <- function(x, component = c("all", "conditional", "zi", "zero_inflated"), verbose = TRUE, ...) {
+get_data.zcpglm <- function(x,
+                            component = c("all", "conditional", "zi", "zero_inflated"),
+                            verbose = TRUE,
+                            ...) {
   component <- match.arg(component)
 
   mf <- stats::model.frame(x)
@@ -233,8 +294,7 @@ get_data.zcpglm <- function(x, component = c("all", "conditional", "zi", "zero_i
     mf_zero <- NULL
   }
 
-  mf <- switch(
-    component,
+  mf <- switch(component,
     "all" = do.call(cbind, .compact_list(list(mf_tweedie, mf_zero))),
     "conditional" = mf_tweedie,
     "zi" = ,
@@ -245,23 +305,26 @@ get_data.zcpglm <- function(x, component = c("all", "conditional", "zi", "zero_i
 }
 
 
-
-
-
-
-
-
-
 # mixed models -------------------------------------------------------------
 
 
 #' @rdname get_data
 #' @export
-get_data.glmmTMB <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
+get_data.glmmTMB <- function(x,
+                             effects = c("all", "fixed", "random"),
+                             component = c("all", "conditional", "zi", "zero_inflated", "dispersion"),
+                             verbose = TRUE,
+                             ...) {
   effects <- match.arg(effects)
   component <- match.arg(component)
 
-  model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
+  model.terms <- find_variables(
+    x,
+    effects = "all",
+    component = "all",
+    flatten = FALSE,
+    verbose = FALSE
+  )
 
   mf <- tryCatch(
     {
@@ -279,20 +342,22 @@ get_data.glmmTMB <- function(x, effects = c("all", "fixed", "random"), component
   mf <- .add_zeroinf_data(x, mf, model.terms$zero_inflated)
   mf <- .add_zeroinf_data(x, mf, model.terms$zero_inflated_random)
 
-  .return_data(x, mf, effects, component, model.terms, verbose = verbose)
+  .return_combined_data(x, mf, effects, component, model.terms, verbose = verbose)
 }
 
 
 
 #' @rdname get_data
 #' @export
-get_data.merMod <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.merMod <- function(x,
+                            effects = c("all", "fixed", "random"),
+                            verbose = TRUE,
+                            ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
+      switch(effects,
         fixed = stats::model.frame(x, fixed.only = TRUE),
         all = stats::model.frame(x, fixed.only = FALSE),
         random = stats::model.frame(x, fixed.only = FALSE)[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
@@ -315,13 +380,15 @@ get_data.merModList <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 
 #' @export
-get_data.MANOVA <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.MANOVA <- function(x,
+                            effects = c("all", "fixed", "random"),
+                            verbose = TRUE,
+                            ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
+      switch(effects,
         fixed = .remove_column(x$input$data, x$input$subject),
         all = x$input$data,
         random = x$input$data[, x$input$subject, drop = FALSE]
@@ -341,15 +408,17 @@ get_data.RM <- get_data.MANOVA
 
 
 #' @export
-get_data.cpglmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.cpglmm <- function(x,
+                            effects = c("all", "fixed", "random"),
+                            verbose = TRUE,
+                            ...) {
   effects <- match.arg(effects)
   dat <- stats::model.frame(x)
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
-        fixed = dat[, find_predictors(x, effects = "fixed", flatten = TRUE), drop = FALSE],
+      switch(effects,
+        fixed = dat[, find_predictors(x, effects = "fixed", flatten = TRUE, verbose = FALSE), drop = FALSE],
         all = dat,
         random = dat[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
       )
@@ -367,15 +436,21 @@ get_data.HLfit <- get_data.cpglmm
 
 
 #' @export
-get_data.glmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.glmm <- function(x,
+                          effects = c("all", "fixed", "random"),
+                          verbose = TRUE,
+                          ...) {
   effects <- match.arg(effects)
   dat <- get_data.default(x, verbose = verbose)
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
-        fixed = dat[, find_predictors(x, effects = "fixed", flatten = TRUE), drop = FALSE],
+      switch(effects,
+        fixed = dat[, find_predictors(x,
+          effects = "fixed",
+          flatten = TRUE,
+          verbose = FALSE
+        ), drop = FALSE],
         all = dat,
         random = dat[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
       )
@@ -389,13 +464,15 @@ get_data.glmm <- function(x, effects = c("all", "fixed", "random"), verbose = TR
 
 
 #' @export
-get_data.mixor <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.mixor <- function(x,
+                           effects = c("all", "fixed", "random"),
+                           verbose = TRUE,
+                           ...) {
   effects <- match.arg(effects)
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
+      switch(effects,
         fixed = stats::model.frame(x),
         all = cbind(stats::model.frame(x), x$id),
         random = data.frame(x$id)
@@ -407,7 +484,7 @@ get_data.mixor <- function(x, effects = c("all", "fixed", "random"), verbose = T
   )
 
   fix_cn <- which(colnames(mf) %in% c("x.id", "x$id"))
-  colnames(mf)[fix_cn] <- deparse(x$call$id)
+  colnames(mf)[fix_cn] <- .safe_deparse(x$call$id)
 
   .prepare_get_data(x, mf, effects, verbose = verbose)
 }
@@ -416,16 +493,18 @@ get_data.mixor <- function(x, effects = c("all", "fixed", "random"), verbose = T
 
 #' @rdname get_data
 #' @export
-get_data.glmmadmb <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.glmmadmb <- function(x,
+                              effects = c("all", "fixed", "random"),
+                              verbose = TRUE,
+                              ...) {
   effects <- match.arg(effects)
 
   fixed_data <- x$frame
-  random_data <- .get_data_from_env(x)[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
+  random_data <- .recover_data_from_environment(x)[, find_random(x, split_nested = TRUE, flatten = TRUE), drop = FALSE]
 
   mf <- tryCatch(
     {
-      switch(
-        effects,
+      switch(effects,
         fixed = fixed_data,
         all = cbind(fixed_data, random_data),
         random = random_data
@@ -436,7 +515,7 @@ get_data.glmmadmb <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects)
+  .prepare_get_data(x, mf, effects, verbose = verbose)
 }
 
 
@@ -462,31 +541,39 @@ get_data.mixed <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 
 #' @export
-get_data.afex_aov <- function(x, ...) {
-  .get_data_from_modelframe(x, x$data$long, "all")
+#' @rdname get_data
+#' @param shape Return long or wide data? Only applicable in repeated measures
+#'   designs.
+get_data.afex_aov <- function(x, shape = c("long", "wide"), ...) {
+  if (!length(attr(x, "within"))) {
+    shape <- "long"
+  } else {
+    shape <- match.arg(shape)
+  }
+  x$data[[shape]]
 }
 
 
 
 #' @export
-get_data.sem <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.sem <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)
-      switch(
-        effects,
-        all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
-        fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
-        random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
+      dat <- .recover_data_from_environment(x)
+      vars <- switch(effects,
+        all = find_variables(x, flatten = TRUE, verbose = FALSE),
+        fixed = find_variables(x, effects = "fixed", flatten = TRUE, verbose = FALSE),
+        random = find_random(x, flatten = TRUE)
       )
+      dat[, intersect(vars, colnames(dat)), drop = FALSE]
     },
     error = function(x) {
       NULL
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf), effects)
+  .prepare_get_data(x, stats::na.omit(mf), effects, verbose = verbose)
 }
 
 
@@ -509,10 +596,13 @@ get_data.lme <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 
 
-#' @importFrom stats model.frame
 #' @rdname get_data
 #' @export
-get_data.MixMod <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated", "dispersion"), verbose = TRUE, ...) {
+get_data.MixMod <- function(x,
+                            effects = c("all", "fixed", "random"),
+                            component = c("all", "conditional", "zi", "zero_inflated", "dispersion"),
+                            verbose = TRUE,
+                            ...) {
   effects <- match.arg(effects)
   component <- match.arg(component)
 
@@ -541,10 +631,17 @@ get_data.MixMod <- function(x, effects = c("all", "fixed", "random"), component
       colnames(fitfram)[ncol(fitfram)] <- x$id_name[1]
 
       # test...
-      fitfram <- .prepare_get_data(x, fitfram, effects)
+      fitfram <- .prepare_get_data(x, fitfram, effects, verbose = verbose)
 
-      model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
-      .return_data(x, mf = fitfram, effects, component, model.terms, verbose = verbose)
+      model.terms <- find_variables(
+        x,
+        effects = "all",
+        component = "all",
+        flatten = FALSE,
+        verbose = FALSE
+      )
+
+      .return_combined_data(x, mf = fitfram, effects, component, model.terms, verbose = verbose)
     },
     error = function(x) {
       NULL
@@ -555,13 +652,12 @@ get_data.MixMod <- function(x, effects = c("all", "fixed", "random"), component
 
 
 #' @export
-get_data.BBmm <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.BBmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
-      switch(
-        effects,
+      dat <- .recover_data_from_environment(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
+      switch(effects,
         all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
         fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
         random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
@@ -572,34 +668,28 @@ get_data.BBmm <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf), effects)
+  .prepare_get_data(x, stats::na.omit(mf), effects, verbose = verbose)
 }
 
 
-
 #' @export
-get_data.glimML <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.glimML <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   dat <- x@data
-  mf <- switch(
-    effects,
+  mf <- switch(effects,
     all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
     fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
     random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
   )
 
-  .prepare_get_data(x, stats::na.omit(mf), effects)
+  .prepare_get_data(x, stats::na.omit(mf), effects, verbose = verbose)
 }
 
 
-
-
-
-
 # sem models -------------------------------------
 
 #' @export
-get_data.lavaan <- function(x, ...) {
+get_data.lavaan <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       .get_S4_data_from_env(x)
@@ -609,22 +699,17 @@ get_data.lavaan <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 #' @export
 get_data.blavaan <- get_data.lavaan
 
 
-
-
-
-
-
 # additive models (gam) -------------------------------------
 
 #' @export
-get_data.vgam <- function(x, ...) {
+get_data.vgam <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       get(x@misc$dataname, envir = parent.frame())[, find_variables(x, flatten = TRUE), drop = FALSE]
@@ -634,21 +719,21 @@ get_data.vgam <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
 #' @export
-get_data.gamm <- function(x, ...) {
+get_data.gamm <- function(x, verbose = TRUE, ...) {
   x <- x$gam
   class(x) <- c(class(x), c("glm", "lm"))
   mf <- stats::model.frame(x)
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
 #' @export
-get_data.gamlss <- function(x, ...) {
+get_data.gamlss <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       elements <- c("mu", "sigma", "nu", "tau")
@@ -676,7 +761,7 @@ get_data.gamlss <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects = "all")
+  .prepare_get_data(x, mf, effects = "all", verbose = verbose)
 }
 
 
@@ -685,9 +770,9 @@ get_data.gamlss <- function(x, ...) {
 # fixed effects and panel regression --------------------------------------
 
 #' @export
-get_data.felm <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.felm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
-  .get_data_from_modelframe(x, stats::model.frame(x), effects)
+  .get_data_from_modelframe(x, stats::model.frame(x), effects, verbose = verbose)
 }
 
 
@@ -696,7 +781,7 @@ get_data.feis <- function(x, effects = c("all", "fixed", "random"), ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
-      .get_data_from_env(x)
+      .recover_data_from_environment(x)
     },
     error = function(x) {
       stats::model.frame(x)
@@ -709,7 +794,7 @@ get_data.feis <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 #' @export
 get_data.fixest <- function(x, ...) {
-  mf <- .get_data_from_env(x)
+  mf <- .recover_data_from_environment(x)
   .get_data_from_modelframe(x, mf, effects = "all")
 }
 
@@ -722,7 +807,22 @@ get_data.feglm <- function(x, ...) {
 
 
 #' @export
-get_data.plm <- function(x, ...) {
+get_data.pgmm <- function(x, verbose = TRUE, ...) {
+  model_terms <- find_variables(x, effects = "all", component = "all", flatten = TRUE)
+  mf <- tryCatch(
+    {
+      .recover_data_from_environment(x)[, model_terms, drop = FALSE]
+    },
+    error = function(x) {
+      NULL
+    }
+  )
+  .prepare_get_data(x, mf, verbose = verbose)
+}
+
+
+#' @export
+get_data.plm <- function(x, verbose = TRUE, ...) {
   mf <- stats::model.frame(x)
   model_terms <- find_variables(x, effects = "all", component = "all", flatten = TRUE)
   cn <- colnames(mf)
@@ -734,7 +834,7 @@ get_data.plm <- function(x, ...) {
 
   # try to get index variables from orignal data
   if (!is.null(index)) {
-    original_data <- .get_data_from_env(x)
+    original_data <- .recover_data_from_environment(x)
     keep <- intersect(index, colnames(original_data))
     if (length(keep)) {
       mf <- cbind(mf, original_data[, keep, drop = FALSE])
@@ -742,12 +842,12 @@ get_data.plm <- function(x, ...) {
     }
   }
 
-  .prepare_get_data(x, mf[, model_terms, drop = FALSE])
+  .prepare_get_data(x, mf[, model_terms, drop = FALSE], verbose = verbose)
 }
 
 
 #' @export
-get_data.wbm <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.wbm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- stats::model.frame(x)
 
@@ -760,7 +860,7 @@ get_data.wbm <- function(x, effects = c("all", "fixed", "random"), ...) {
   resp.col <- which(colnames(mf) == find_response(x))
   mf <- mf[, c(resp.col, (1:ncol(mf))[-resp.col])]
 
-  .prepare_get_data(x, stats::na.omit(mf), effects)
+  .prepare_get_data(x, stats::na.omit(mf), effects, verbose = verbose)
 }
 
 
@@ -769,7 +869,7 @@ get_data.wbgee <- get_data.wbm
 
 
 #' @export
-get_data.ivreg <- function(x, ...) {
+get_data.ivreg <- function(x, verbose = TRUE, ...) {
   mf <- stats::model.frame(x)
   cn <- clean_names(colnames(mf))
   ft <- find_variables(x, flatten = TRUE)
@@ -781,7 +881,7 @@ get_data.ivreg <- function(x, ...) {
   } else {
     final_mf <- tryCatch(
       {
-        dat <- .get_data_from_env(x)
+        dat <- .recover_data_from_environment(x)
         cbind(mf, dat[, remain, drop = FALSE])
       },
       error = function(x) {
@@ -790,7 +890,7 @@ get_data.ivreg <- function(x, ...) {
     )
   }
 
-  .prepare_get_data(x, stats::na.omit(final_mf))
+  .prepare_get_data(x, stats::na.omit(final_mf), verbose = verbose)
 }
 
 #' @export
@@ -798,13 +898,13 @@ get_data.iv_robust <- get_data.ivreg
 
 
 #' @export
-get_data.ivprobit <- function(x, ...) {
-  .prepare_get_data(x, stats::na.omit(as.data.frame(x$mr1)))
+get_data.ivprobit <- function(x, verbose = TRUE, ...) {
+  .prepare_get_data(x, stats::na.omit(as.data.frame(x$mr1)), verbose = verbose)
 }
 
 
 #' @export
-get_data.bife <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.bife <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- as.data.frame(x$data)
 
@@ -814,7 +914,7 @@ get_data.bife <- function(x, effects = c("all", "fixed", "random"), ...) {
     mf <- mf[, setdiff(colnames(mf), unique(find_random(x, split_nested = TRUE, flatten = TRUE))), drop = FALSE]
   }
 
-  .prepare_get_data(x, stats::na.omit(mf), effects)
+  .prepare_get_data(x, stats::na.omit(mf), effects, verbose = verbose)
 }
 
 
@@ -830,9 +930,13 @@ get_data.bife <- function(x, effects = c("all", "fixed", "random"), ...) {
 
 #' @rdname get_data
 #' @export
-get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated"), verbose = TRUE, ...) {
-  effects <- match.arg(effects)
-  component <- match.arg(component)
+get_data.brmsfit <- function(x,
+                             effects = "all",
+                             component = "all",
+                             verbose = TRUE,
+                             ...) {
+  effects <- match.arg(effects, choices = c("all", "fixed", "random"))
+  component <- match.arg(component, choices = c("all", .all_elements()))
 
   model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
   mf <- stats::model.frame(x)
@@ -843,9 +947,9 @@ get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component
     if (!.is_empty_object(rs)) model.terms$random <- c(rs, model.terms$random)
   }
 
-  .return_data(
+  .return_combined_data(
     x,
-    .prepare_get_data(x, mf, effects = effects),
+    .prepare_get_data(x, mf, effects = effects, verbose = verbose),
     effects,
     component,
     model.terms,
@@ -858,13 +962,21 @@ get_data.brmsfit <- function(x, effects = c("all", "fixed", "random"), component
 
 #' @rdname get_data
 #' @export
-get_data.stanreg <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
+get_data.stanreg <- function(x,
+                             effects = c("all", "fixed", "random"),
+                             verbose = TRUE,
+                             ...) {
   effects <- match.arg(effects)
 
-  model.terms <- find_variables(x, effects = "all", component = "all", flatten = FALSE)
+  model.terms <- find_variables(x,
+    effects = "all",
+    component = "all",
+    flatten = FALSE
+  )
+
   mf <- stats::model.frame(x)
 
-  .return_data(
+  .return_combined_data(
     x,
     .prepare_get_data(x, mf, effects = effects, verbose = verbose),
     effects,
@@ -886,7 +998,7 @@ get_data.BFBayesFactor <- function(x, ...) {
 
 #' @rdname get_data
 #' @export
-get_data.MCMCglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
+get_data.MCMCglmm <- function(x, effects = c("all", "fixed", "random"), verbose = TRUE, ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
@@ -899,8 +1011,7 @@ get_data.MCMCglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
       mf <- env_dataframes[matchframe][1]
       if (!is.na(mf)) {
         dat <- get(mf)
-        switch(
-          effects,
+        switch(effects,
           fixed = dat[, setdiff(colnames(dat), find_random(x, flatten = TRUE)), drop = FALSE],
           all = dat,
           random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
@@ -914,13 +1025,13 @@ get_data.MCMCglmm <- function(x, effects = c("all", "fixed", "random"), ...) {
     }
   )
 
-  .prepare_get_data(x, mf, effects = effects)
+  .prepare_get_data(x, mf, effects = effects, verbose = verbose)
 }
 
 
 
 #' @export
-get_data.stanmvreg <- function(x, ...) {
+get_data.stanmvreg <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       out <- data.frame()
@@ -935,14 +1046,10 @@ get_data.stanmvreg <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
-
-
-
-
 # mfx models ------------------------------------------------------
 
 #' @export
@@ -976,12 +1083,16 @@ get_data.negbinmfx <- get_data.betamfx
 
 
 
-
-
-
 # other models ------------------------------------------------------
 
 
+#' @export
+get_data.svy_vglm <- function(x, verbose = TRUE, ...) {
+  mf <- x$design$variables[find_variables(x, flatten = TRUE)]
+  .prepare_get_data(x, mf, verbose = verbose)
+}
+
+
 #' @export
 get_data.mediate <- function(x, ...) {
   d1 <- get_data(x$model.m)
@@ -1026,7 +1137,7 @@ get_data.averaging <- function(x, ...) {
   if (is.null(mf)) {
     mf <- tryCatch(
       {
-        .get_data_from_env(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
+        .recover_data_from_environment(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
       },
       error = function(x) {
         NULL
@@ -1079,10 +1190,10 @@ get_data.mcmc.list <- function(x, ...) {
 
 
 #' @export
-get_data.DirichletRegModel <- function(x, ...) {
+get_data.DirichletRegModel <- function(x, verbose = TRUE, ...) {
   mf <- x$data
   resp <- sapply(x$data, inherits, "DirichletRegData")
-  .prepare_get_data(x, mf[!resp])
+  .prepare_get_data(x, mf[!resp], verbose = verbose)
 }
 
 
@@ -1126,9 +1237,8 @@ get_data.LORgee <- function(x, effects = c("all", "fixed", "random"), ...) {
   effects <- match.arg(effects)
   mf <- tryCatch(
     {
-      dat <- .get_data_from_env(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
-      switch(
-        effects,
+      dat <- .recover_data_from_environment(x)[, find_variables(x, flatten = TRUE), drop = FALSE]
+      switch(effects,
         all = dat[, find_variables(x, flatten = TRUE), drop = FALSE],
         fixed = dat[, find_variables(x, effects = "fixed", flatten = TRUE), drop = FALSE],
         random = dat[, find_random(x, flatten = TRUE), drop = FALSE]
@@ -1177,12 +1287,12 @@ get_data.gbm <- function(x, ...) {
 
 
 #' @export
-get_data.tobit <- function(x, ...) {
-  dat <- .get_data_from_env(x)
-  ft <- find_variables(x, flatten = TRUE)
+get_data.tobit <- function(x, verbose = TRUE, ...) {
+  dat <- .recover_data_from_environment(x)
+  ft <- find_variables(x, flatten = TRUE, verbose = FALSE)
   remain <- intersect(ft, colnames(dat))
 
-  .prepare_get_data(x, stats::na.omit(dat[, remain, drop = FALSE]))
+  .prepare_get_data(x, stats::na.omit(dat[, remain, drop = FALSE]), verbose = verbose)
 }
 
 
@@ -1200,7 +1310,7 @@ get_data.clmm2 <- function(x, ...) {
       }
 
       data_complete <- cbind(data_complete, x$grFac)
-      colnames(data_complete)[ncol(data_complete)] <- unlist(.find_random_effects(x, f = find_formula(x), split_nested = TRUE))
+      colnames(data_complete)[ncol(data_complete)] <- unlist(.find_random_effects(x, f = find_formula(x, verbose = FALSE), split_nested = TRUE))
 
       data_complete
     },
@@ -1238,15 +1348,15 @@ get_data.clm2 <- function(x, ...) {
 
 
 #' @export
-get_data.bracl <- function(x, ...) {
+get_data.bracl <- function(x, verbose = TRUE, ...) {
   mf <- stats::model.frame(x)
-  suppressWarnings(.prepare_get_data(x, mf))
+  suppressWarnings(.prepare_get_data(x, mf, verbose = verbose))
 }
 
 
 
 #' @export
-get_data.mlogit <- function(x, ...) {
+get_data.mlogit <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       as.data.frame(stats::model.frame(x))
@@ -1256,23 +1366,23 @@ get_data.mlogit <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, mf)
+  .prepare_get_data(x, mf, verbose = verbose)
 }
 
 
 
 #' @export
-get_data.rma <- function(x, ...) {
+get_data.rma <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
-      .get_data_from_env(x)
+      .recover_data_from_environment(x)
     },
     error = function(x) {
       NULL
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 
@@ -1281,7 +1391,7 @@ get_data.metaplus <- get_data.rma
 
 
 #' @export
-get_data.meta_random <- function(x, ...) {
+get_data.meta_random <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       x$data$data
@@ -1291,12 +1401,12 @@ get_data.meta_random <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 
 #' @export
-get_data.meta_bma <- function(x, ...) {
+get_data.meta_bma <- function(x, verbose = TRUE, ...) {
   mf <- tryCatch(
     {
       x$meta$fixed$data$data
@@ -1306,7 +1416,7 @@ get_data.meta_bma <- function(x, ...) {
     }
   )
 
-  .prepare_get_data(x, stats::na.omit(mf))
+  .prepare_get_data(x, stats::na.omit(mf), verbose = verbose)
 }
 
 
@@ -1318,6 +1428,12 @@ get_data.meta_fixed <- get_data.meta_random
 get_data.ivFixed <- get_data.rma
 
 
+#' @export
+get_data.bfsl <- function(x, ...) {
+  as.data.frame(x$data[c("x", "y", "sd_x", "sd_y")])
+}
+
+
 #' @export
 get_data.mipo <- function(x, ...) {
   tryCatch(
@@ -1336,10 +1452,7 @@ get_data.mipo <- function(x, ...) {
 get_data.htest <- function(x, ...) {
   out <- NULL
   if (!is.null(x$data.name)) {
-    for (parent_level in 1:5) {
-      out <- .retrieve_htest_data(x, parent_level)
-      if (!is.null(out)) break
-    }
+    out <- .retrieve_htest_data(x)
   }
   out
 }
