@@ -7,13 +7,13 @@
 #' @param iterations Number of posterior draws.
 #' @param progress Display progress.
 #' @param summary Logical, indicates whether the full posterior samples
-#'   (\code{summary = FALSE})) or the summarized centrality indices of
-#'   the posterior samples (\code{summary = TRUE})) should be returned as
+#'   (`summary = FALSE`)) or the summarized centrality indices of
+#'   the posterior samples (`summary = TRUE`)) should be returned as
 #'   estimates.
 #' @param centrality Only for models with posterior samples, and when
-#'   \code{summary = TRUE}. In this case, \code{centrality = "mean"} would
+#'   `summary = TRUE`. In this case, `centrality = "mean"` would
 #'   calculate means of posterior samples for each parameter, while
-#'   \code{centrality = "median"} would use the more robust median value as
+#'   `centrality = "median"` would use the more robust median value as
 #'   measure of central tendency.
 #' @param verbose Toggle messages and warnings.
 #' @param ... Currently not used.
@@ -23,20 +23,20 @@
 #' @inheritParams find_predictors
 #'
 #' @return The posterior samples from the requested parameters as data frame.
-#'   If \code{summary = TRUE}, returns a data frame with two columns: the
-#'   parameter names and the related point estimates (based on \code{centrality}).
+#'   If `summary = TRUE`, returns a data frame with two columns: the
+#'   parameter names and the related point estimates (based on `centrality`).
 #'
 #' @details In most cases when models either return different "effects" (fixed,
 #' random) or "components" (conditional, zero-inflated, ...), the arguments
-#' \code{effects} and \code{component} can be used.
+#' `effects` and `component` can be used.
 #'
 #' @section BFBayesFactor Models:
-#' Note that for \code{BFBayesFactor} models (from the \pkg{BayesFactor}
+#' Note that for `BFBayesFactor` models (from the \pkg{BayesFactor}
 #' package), posteriors are only extracted from the first numerator model (i.e.,
-#' \code{model[1]}). If you want to apply some function \code{foo()} to another
-#' model stored in the \code{BFBayesFactor} object, index it directly, e.g.
-#' \code{foo(model[2])}, \code{foo(1/model[5])}, etc.
-#' See also \code{\link[bayestestR]{weighted_posteriors}}.
+#' `model[1]`). If you want to apply some function `foo()` to another
+#' model stored in the `BFBayesFactor` object, index it directly, e.g.
+#' `foo(model[2])`, `foo(1/model[5])`, etc.
+#' See also [bayestestR::weighted_posteriors()].
 #'
 #' @examples
 #' data(mtcars)
@@ -47,18 +47,17 @@ get_parameters.BGGM <- function(x,
                                 component = c("correlation", "conditional", "intercept", "all"),
                                 summary = FALSE, centrality = "mean",
                                 ...) {
-  if (!requireNamespace("BGGM", quietly = TRUE)) {
-    stop("Package 'BGGM' required for this function to work. Please install it.")
-  }
-
-  out <- as.data.frame(BGGM::posterior_samples(x))
+  # installed?
+  # check_if_installed("BGGM")
+  #
+  # out <- as.data.frame(BGGM::posterior_samples(x))
+  out <- as.data.frame(.bggm_posterior_samples(x))
   intercepts <- grepl("_\\(Intercept\\)$", colnames(out))
   correlations <- grepl("(.*)--(.*)", colnames(out))
   conditional <- !intercepts & !correlations
 
   component <- match.arg(component)
-  out <- switch(
-    component,
+  out <- switch(component,
     "conditional" = out[, conditional, drop = FALSE],
     "correlation" = out[, correlations, drop = FALSE],
     "intercept" = out[, intercepts, drop = FALSE],
@@ -111,9 +110,8 @@ get_parameters.BFBayesFactor <- function(x,
                                          summary = FALSE,
                                          centrality = "mean",
                                          ...) {
-  if (!requireNamespace("BayesFactor", quietly = TRUE)) {
-    stop("This function requires package `BayesFactor` to work. Please install it.")
-  }
+  # installed?
+  check_if_installed("BayesFactor")
 
   effects <- match.arg(effects)
   component <- match.arg(component)
@@ -122,10 +120,10 @@ get_parameters.BFBayesFactor <- function(x,
   # check if valid model was indexed...
 
   if (length(x@numerator) > 1 ||
-      !xor(
-        x@denominator@shortName == "Intercept only",
-        grepl("^(Null|Indep)", x@denominator@shortName)
-      )) {
+    !xor(
+      x@denominator@shortName == "Intercept only",
+      grepl("^(Null|Indep)", x@denominator@shortName)
+    )) {
     if (verbose) {
       message(
         "Multiple `BFBayesFactor` models detected - posteriors are extracted from the first numerator model.\n",
@@ -143,11 +141,10 @@ get_parameters.BFBayesFactor <- function(x,
         BayesFactor::posterior(x, iterations = iterations, progress = progress, index = 1, ...)
       ))
 
-    out <- switch(
-      bf_type,
+    out <- switch(bf_type,
       "correlation" = data.frame("rho" = as.numeric(posteriors$rho)),
-      "ttest1" = data.frame("Difference" = x@numerator[[1]]@prior$mu - as.numeric(posteriors[, 1])),
-      "ttest2" = data.frame("Difference" = x@numerator[[1]]@prior$mu - as.numeric(posteriors[, 2])),
+      "ttest1" = data.frame("Difference" = as.numeric(posteriors[, 1]) - x@numerator[[1]]@prior$mu),
+      "ttest2" = data.frame("Difference" = as.numeric(posteriors[, 2]) - x@numerator[[1]]@prior$mu),
       "meta" = data.frame("Effect" = as.numeric(posteriors$delta)),
       "linear" = .get_bf_posteriors(posteriors, params),
       NULL
@@ -159,7 +156,7 @@ get_parameters.BFBayesFactor <- function(x,
     colnames(posteriors) <- "p"
     out <- posteriors
   } else if (bf_type == "xtable") {
-    data <- get_data(x)
+    data <- get_data(x, verbose = verbose)
     N <- sum(data)
     cells <- prod(dim(data))
     posts <- as.data.frame(as.matrix(suppressMessages(
@@ -220,23 +217,21 @@ get_parameters.stanmvreg <- function(x,
 #' @rdname get_parameters.BGGM
 #' @export
 get_parameters.brmsfit <- function(x,
-                                   effects = c("fixed", "random", "all"),
-                                   component = c("all", "conditional", "location", "distributional", "auxiliary", "zi", "zero_inflated", "dispersion", "simplex", "sigma", "smooth_terms"),
+                                   effects = "fixed",
+                                   component = "all",
                                    parameters = NULL,
                                    summary = FALSE,
                                    centrality = "mean",
                                    ...) {
-  effects <- match.arg(effects)
-  component <- match.arg(component)
+  effects <- match.arg(effects, choices = c("all", "fixed", "random"))
+  component <- match.arg(component, choices = c("all", .all_elements()))
 
   if (is_multivariate(x)) {
     parms <- find_parameters(x, flatten = FALSE, parameters = parameters)
     elements <- .get_elements(effects, component)
-    ## TODO remove "optional = FALSE" in a future update
-    out <- as.data.frame(x, optional = FALSE)[unlist(lapply(parms, function(i) i[elements]))]
+    out <- as.data.frame(x)[unlist(lapply(parms, function(i) i[elements]))]
   } else {
-    ## TODO remove "optional = FALSE" in a future update
-    out <- as.data.frame(x, optional = FALSE)[.get_parms_data(x, effects, component, parameters)]
+    out <- as.data.frame(x)[.get_parms_data(x, effects, component, parameters)]
   }
 
   if (isTRUE(summary)) {
@@ -313,8 +308,7 @@ get_parameters.bayesx <- function(x,
     stringsAsFactors = FALSE
   )
 
-  params <- switch(
-    component,
+  params <- switch(component,
     "all" = rbind(fixed_dat, smooth_dat),
     "conditional" = fixed_dat,
     "smooth_terms" = smooth_dat
@@ -401,6 +395,22 @@ get_parameters.bayesQR <- function(x,
 
 
 
+#' @export
+get_parameters.blrm <- function(x,
+                                parameters = NULL,
+                                summary = FALSE,
+                                centrality = "mean",
+                                ...) {
+  out <- as.data.frame(x$draws)
+  out <- out[.get_parms_data(x, "all", "all", parameters)]
+  if (isTRUE(summary)) {
+    out <- .summary_of_posteriors(out, centrality = centrality)
+  }
+  out
+}
+
+
+
 #' @rdname get_parameters.BGGM
 #' @export
 get_parameters.sim.merMod <- function(x,
@@ -448,13 +458,11 @@ get_parameters.sim <- function(x,
 # helper -----------------------
 
 
-#' @importFrom stats median
 .summary_of_posteriors <- function(out, centrality = "mean", ...) {
-  s <- switch(
-    centrality,
-    "mean" = sapply(out, mean),
-    "median" = sapply(out, stats::median),
-    sapply(out, mean)
+  s <- switch(centrality,
+    "mean" = sapply(out, mean, na.rm = TRUE),
+    "median" = sapply(out, stats::median, na.rm = TRUE),
+    sapply(out, mean, na.rm = TRUE)
   )
   data.frame(
     Parameter = names(s),
@@ -477,3 +485,111 @@ get_parameters.sim <- function(x,
   unlist(find_parameters(x, effects = "all", component = "all", flatten = FALSE, parameters = parameters)[elements])
 }
 
+
+
+
+
+# use temporarily code from BGGM package, as long as that package is archived on CRAN
+
+.bggm_posterior_samples <- function(object, ...) {
+  if (methods::is(object, "estimate") | methods::is(object, "explore")) {
+    if (!methods::is(object, "default")) {
+      stop("object most be from 'estimate' or 'explore'")
+    }
+    p <- object$p
+    pcors_total <- p * (p - 1) * 0.5
+    I_p <- diag(p)
+    iter <- object$iter
+    pcor_samples <- matrix(object$post_samp$pcors[, , 51:(iter + 50)][upper.tri(I_p)],
+      nrow = iter,
+      ncol = pcors_total,
+      byrow = TRUE
+    )
+    cn <- colnames(object$Y)
+    if (is.null(cn)) {
+      col_names <- sapply(1:p, function(x) paste(1:p, x, sep = "--"))[upper.tri(I_p)]
+    } else {
+      col_names <- sapply(cn, function(x) paste(cn, x, sep = "--"))[upper.tri(I_p)]
+    }
+    colnames(pcor_samples) <- col_names
+    posterior_samples <- pcor_samples
+    if (!is.null(object$formula)) {
+      if (ncol(object$X) == 1) {
+        beta_terms <- "(Intercept)"
+      } else {
+        beta_terms <- colnames(object$X)
+      }
+      n_beta_terms <- length(beta_terms)
+      beta_samples <- object$post_samp$beta
+      if (is.null(cn)) {
+        col_names <- 1:p
+      } else {
+        col_names <- cn
+      }
+      beta_start <- matrix(beta_samples[1:n_beta_terms, 1, 51:(iter + 50)],
+        nrow = iter,
+        n_beta_terms,
+        byrow = TRUE
+      )
+      colnames(beta_start) <- paste0(
+        col_names[1], "_",
+        beta_terms
+      )
+      for (i in 2:p) {
+        beta_i <- matrix(beta_samples[1:n_beta_terms, i, 51:(iter + 50)],
+          nrow = iter,
+          n_beta_terms,
+          byrow = TRUE
+        )
+        colnames(beta_i) <- paste0(col_names[i], "_", beta_terms)
+        beta_start <- cbind(beta_start, beta_i)
+      }
+      posterior_samples <- cbind(posterior_samples, beta_start)
+    }
+  } else if (methods::is(object, "var_estimate")) {
+    if (!methods::is(object, "default")) {
+      stop("object most be from 'var_estimate'")
+    }
+    p <- object$p
+    pcors_total <- p * (p - 1) * 0.5
+    I_p <- diag(p)
+    iter <- object$iter
+    pcor_samples <- matrix(object$fit$pcors[, , 51:(iter + 50)][upper.tri(I_p)],
+      nrow = iter,
+      ncol = pcors_total,
+      byrow = TRUE
+    )
+    cn <- colnames(object$Y)
+    if (is.null(cn)) {
+      col_names <- sapply(1:p, function(x) paste(1:p, x, sep = "--"))[upper.tri(I_p)]
+    } else {
+      col_names <- sapply(cn, function(x) paste(cn, x, sep = "--"))[upper.tri(I_p)]
+    }
+    colnames(pcor_samples) <- col_names
+    posterior_samples <- pcor_samples
+    n_beta_terms <- nrow(object$beta_mu)
+    beta_samples <- object$fit$beta
+    col_names <- colnames(object$Y)
+    beta_terms <- colnames(object$X)
+    beta_start <- matrix(beta_samples[1:n_beta_terms, 1, 51:(iter + 50)],
+      nrow = iter,
+      n_beta_terms,
+      byrow = TRUE
+    )
+    colnames(beta_start) <- paste0(col_names[1], "_", beta_terms)
+    for (i in 2:p) {
+      beta_i <- matrix(beta_samples[1:n_beta_terms, i, 51:(iter + 50)],
+        nrow = iter,
+        n_beta_terms,
+        byrow = TRUE
+      )
+      colnames(beta_i) <- paste0(col_names[i], "_", beta_terms)
+      beta_start <- cbind(beta_start, beta_i)
+    }
+    posterior_samples <- cbind(posterior_samples, beta_start)
+  } else {
+    stop("object class not currently supported")
+  }
+
+  posterior_samples
+}
