@@ -24,11 +24,17 @@ get_priors <- function(x, ...) {
 }
 
 
+
+# =========================================================================
+# RSTANARM ----------------------------------------------------------------
+# =========================================================================
+
+
+
 #' @export
-get_priors.stanreg <- function(x, ...) {
-  if (!requireNamespace("rstanarm", quietly = TRUE)) {
-    stop("To use this function, please install package 'rstanarm'.")
-  }
+get_priors.stanreg <- function(x, verbose = TRUE, ...) {
+  # installed?
+  check_if_installed("rstanarm")
 
   ps <- rstanarm::prior_summary(x)
 
@@ -73,7 +79,23 @@ get_priors.stanreg <- function(x, ...) {
     prior_info$adjusted_scale[flat] <- NA
   }
 
-  prior_info$parameter <- find_parameters(x)$conditional
+
+  params <- find_parameters(x, parameters = "^(?!(R2|log-fit_ratio))")$conditional
+
+  # this is a particular fix for the "R2" prior, which conveys prior
+  # information about *all* the parameters. In this case, number of
+  # parameters doesn't match number of priors
+
+  if (length(params) != nrow(prior_info)) {
+    if (length(params) == 1) {
+      prior_info$parameter <- "(Intercept)"
+    } else if ("R2" %in% prior_info$dist) {
+      prior_info$parameter <- prior_info$dist
+      prior_info$parameter[prior_info$dist != "R2"] <- "(Intercept)"
+    }
+  } else {
+    prior_info$parameter <- params
+  }
   prior_info <- prior_info[, intersect(c("parameter", "dist", "location", "scale", "adjusted_scale"), colnames(prior_info))]
 
   colnames(prior_info) <- gsub("dist", "distribution", colnames(prior_info))
@@ -97,9 +119,9 @@ get_priors.stanreg <- function(x, ...) {
 
 #' @export
 get_priors.stanmvreg <- function(x, ...) {
-  if (!requireNamespace("rstanarm", quietly = TRUE)) {
-    stop("To use this function, please install package 'rstanarm'.")
-  }
+  # installed?
+  check_if_installed("rstanarm")
+
 
   ps <- rstanarm::prior_summary(x)
 
@@ -152,70 +174,223 @@ get_priors.stanmvreg <- function(x, ...) {
 }
 
 
+# =========================================================================
+# BRMS ----------------------------------------------------------------
+# =========================================================================
+
 #' @rdname get_priors
 #' @export
 get_priors.brmsfit <- function(x, verbose = TRUE, ...) {
-  ## TODO needs testing for edge cases - check if "coef"-column is
-  # always empty for intercept-class
-  x$prior$coef[x$prior$class == "Intercept"] <- "(Intercept)"
+  info <- as.data.frame(.print_brmsprior_preparation(x$prior))
+  info$Parameter <- .match_brms_priors_to_params(info)
 
+  priors <- data.frame(Parameter = info$Parameter)
 
-  # get default prior for all parameters, if defined
-  def_prior_b <- which(x$prior$prior != "" & x$prior$class == "b" & x$prior$coef == "")
+  # Format the prior string ------------------------------------
+  priors$Distribution <- gsub(
+    "(.*)\\(.*", "\\1",
+    ifelse(info$prior == "(flat)",
+      "uniform",
+      info$prior
+    )
+  )
+  priors$Distribution[priors$Distribution == "lkj_corr_cholesky"] <- "lkj"
 
-  # check which parameters have a default prior
-  need_def_prior <- which(x$prior$prior == "" & x$prior$class == "b" & x$prior$coef != "")
+  # Initialize empty
+  priors$Location <- NA
+  priors$Scale <- NA
+  priors$df <- NA
 
-  if (!.is_empty_object(def_prior_b) && !.is_empty_object(need_def_prior)) {
-    x$prior$prior[need_def_prior] <- x$prior$prior[def_prior_b]
+  # student_t(df, location, scale)
+  is_student_t <- priors$Distribution == "student_t"
+  priors$Location[is_student_t] <- gsub("(.*)\\((.*)\\,(.*)\\,(.*)\\)", "\\3", info$prior[is_student_t])
+  priors$Scale[is_student_t] <- gsub("(.*)\\((.*)\\,(.*)\\,(.*)\\)", "\\4", info$prior[is_student_t])
+  priors$df[is_student_t] <- gsub("(.*)\\((.*)\\,(.*)\\,(.*)\\)", "\\2", info$prior[is_student_t])
+
+  # normal(location, scale)
+  is_normal <- priors$Distribution == "normal"
+  priors$Location[is_normal] <- gsub("(.*)\\((.*)\\,(.*)\\)", "\\2", info$prior[is_normal])
+  priors$Scale[is_normal] <- gsub("(.*)\\((.*)\\,(.*)\\)", "\\3", info$prior[is_normal])
+
+  # lkj(eta)
+  is_lkj <- priors$Distribution == "lkj"
+  priors$Location[is_lkj] <- gsub("(.*)\\((.*)\\)", "\\2", info$prior[is_lkj])
+
+  # Transform to numeric
+  priors$Location <- as.numeric(priors$Location)
+  priors$Scale <- as.numeric(priors$Scale)
+  priors$df <- as.numeric(priors$df)
+
+  # Get parameters
+  params <- find_parameters(x, ..., flatten = TRUE)
+
+  # remove random
+  params <- params[!grepl("^r_(.*)\\]$", params)]
+  params <- params[!grepl("^r_(.*)\\.\\d$", params)]
+
+  # Loop through all parameters and try to retrieve its correct prior
+  out <- data.frame()
+  for (p in params) {
+    subset <- priors[priors$Parameter == p, ]
+
+    # If nothing corresponding directly to the parameter...
+    if (nrow(subset) == 0) {
+      # Special treatment for cor_*
+      subset <- priors[sapply(priors$Parameter, grepl, x = p), ]
+
+      # If still empty, make empty df
+      if (nrow(subset) == 0) {
+        subset <- stats::setNames(data.frame(t(rep(NA, 5))), c("Parameter", "Distribution", "Location", "Scale", "df"))
+      }
+    }
+
+    # Rbind the stuff
+    subset$Parameter <- p
+    out <- rbind(out, subset)
+  }
+
+  row.names(out) <- NULL
+  attr(out, "priors") <- info
+
+  out
+}
+
+# Utils -------
+
+.print_brmsprior_preparation <- function(x) {
+  # This function is taken from brms:::print.brmsprior
+  # which adds information using private functions upon printing
+  # but doesn't return it
+  check_if_installed("brms")
+
+  .stan_base_prior <- function(prior) {
+    stopifnot(length(unique(prior$class)) <= 1)
+    take <- with(prior, !nzchar(coef) & nzchar(prior))
+    prior <- prior[take, ]
+    if (!NROW(prior)) {
+      return("")
+    }
+    vars <- c("group", "nlpar", "dpar", "resp", "class")
+    for (v in vars) {
+      take <- nzchar(prior[[v]])
+      if (any(take)) {
+        prior <- prior[take, ]
+      }
+    }
+    stopifnot(NROW(prior) == 1)
+    prior$prior
+  }
+
+  .find_rows <- function(x, ..., ls = list(), fun = "%in%") {
+    x <- as.data.frame(x)
+    if (!nrow(x)) {
+      return(logical(0))
+    }
+    out <- rep(TRUE, nrow(x))
+    ls <- c(ls, list(...))
+    if (!length(ls)) {
+      return(out)
+    }
+    if (is.null(names(ls))) {
+      stop("Argument 'ls' must be named.")
+    }
+    for (name in names(ls)) {
+      out <- out & brms::do_call(fun, list(x[[name]], ls[[name]]))
+    }
+    out
   }
 
 
-  # get default prior for all parameters, if defined
-  def_prior_intercept <- which(x$prior$prior != "" & x$prior$class == "Intercept" & x$prior$coef == "")
-
-  # check which parameters have a default prior
-  need_def_prior <- which(x$prior$prior == "" & x$prior$class == "Intercept" & x$prior$coef != "")
-
-  if (!.is_empty_object(def_prior_intercept) && !.is_empty_object(need_def_prior)) {
-    x$prior$prior[need_def_prior] <- x$prior$prior[def_prior_intercept]
+  stopifnot(brms::is.brmsprior(x))
+  if (is.null(x$source)) {
+    x$source <- "(unknown)"
   }
-
-
-  prior_info <- x$prior[x$prior$coef != "" & x$prior$class %in% c("b", "(Intercept)"), ]
-  # find additional components, avoid duplicated coef-names
-  components <- prior_info$dpar != ""
-  prior_info$dpar[components] <- paste0(prior_info$dpar[components], "_")
-  prior_info$coef <- paste0(prior_info$dpar, prior_info$coef)
-
-  prior_info$Distribution <- gsub("(.*)\\(.*", "\\1", prior_info$prior)
-  prior_info$Location <- gsub("(.*)\\((.*)\\,(.*)", "\\2", prior_info$prior)
-  prior_info$Scale <- gsub("(.*)\\,(.*)\\)(.*)", "\\2", prior_info$prior)
-  prior_info$Parameter <- prior_info$coef
-
-  prior_info <- prior_info[, c("Parameter", "Distribution", "Location", "Scale")]
-
-  pinfo <- as.data.frame(lapply(prior_info, function(x) {
-    if (.is_numeric_character(x)) {
-      as.numeric(as.character(x))
+  x$source[!nzchar(x$source)] <- "(unknown)"
+  # column names to vectorize over
+  cols <- c("group", "nlpar", "dpar", "resp", "class")
+  empty_strings <- rep("", 4)
+  for (i in which(!nzchar(x$prior))) {
+    ls <- x[i, cols]
+    ls <- rbind(ls, c(empty_strings, ls$class))
+    ls <- as.list(ls)
+    # sub_prior <- subset2(x, ls = ls)
+    sub_prior <- x[.find_rows(x, ls = ls, fun = "%in%"), , drop = FALSE]
+    base_prior <- .stan_base_prior(sub_prior)
+    if (nzchar(base_prior)) {
+      x$prior[i] <- base_prior
+      x$source[i] <- "(vectorized)"
     } else {
-      as.character(x)
+      x$prior[i] <- "(flat)"
     }
-  }), stringsAsFactors = FALSE)
-
-  if (.is_empty_string(pinfo$Distribution)) {
-    if (verbose) {
-      print_color("Model was fitted with uninformative (flat) priors!\n", "red")
-    }
-    pinfo$Distribution <- "uniform"
-    pinfo$Location <- 0
-    pinfo$Scale <- NA
   }
-
-  pinfo
+  x
 }
 
 
+.match_brms_priors_to_params <- function(prior_summary) {
+  # Rename for easier manipulation
+  pr <- prior_summary
+
+  # Initialize empty string
+  p <- rep("", nrow(pr))
+
+  # class == Intercept -------------------------
+  p <- ifelse(pr$class == "Intercept",
+    paste0(
+      "b",
+      ifelse(pr$dpar != "", paste0("_", pr$dpar), ""),
+      "_Intercept"
+    ),
+    p
+  )
+
+  # class == b ------------------------------
+  # Are there other possible parameters?
+  p <- ifelse(
+    pr$class == "b",
+    paste0(
+      "b_",
+      ifelse(pr$dpar != "", paste0(pr$dpar, "_"), ""),
+      pr$coef
+    ),
+    p
+  )
+
+  # class == L ------------------------------
+  p <- ifelse(pr$class == "L", paste0("cor_", pr$group, "_"), p)
+
+
+  # class == sigma ------------------------------
+  # TODO: I only saw it alone, but possibly can have other parameters
+  p <- ifelse(pr$class == "sigma", "sigma", p)
+
+  # class == sd  -------------------------------
+  p <- ifelse(
+    pr$class == "sd",
+    paste0(
+      "sd_",
+      pr$group,
+      "__",
+      ifelse(pr$dpar != "", paste0(pr$dpar, "_"), ""),
+      pr$coef
+    ),
+    p
+  )
+
+  # class == sds  -------------------------------
+  # TODO: Fix coef for sds_
+  # TODO: Fix beta for smooth term (bs_coef instead of b_coef)
+  # p <- ifelse(
+  #   pr$class == "sds",
+  #   paste0("sds", ifelse(pr$coef  != "", paste0("_", pr$coef), "")),
+  #   p)
+
+  p
+}
+
+# =========================================================================
+# BCPLM ----------------------------------------------------------------
+# =========================================================================
 
 #' @export
 get_priors.bcplm <- function(x, ...) {
@@ -237,6 +412,10 @@ get_priors.bcplm <- function(x, ...) {
 }
 
 
+# =========================================================================
+# meta -------------------------------------------------------------
+# =========================================================================
+
 
 #' @export
 get_priors.meta_random <- function(x, ...) {
@@ -291,15 +470,25 @@ get_priors.meta_fixed <- function(x, ...) {
 }
 
 
+.fix_metabma_priorname <- function(x) {
+  x$Distribution <- gsub("t", "Student's t", x$Distribution, fixed = TRUE)
+  x$Distribution <- gsub("norm", "Normal", x$Distribution, fixed = TRUE)
+  x$Distribution <- gsub("invgamma", "Inverse gamma", x$Distribution, fixed = TRUE)
+  x
+}
+
+
+# =========================================================================
+# BayesFactor -------------------------------------------------------------
+# =========================================================================
+
 
-#' @importFrom utils tail
 #' @export
 get_priors.BFBayesFactor <- function(x, ...) {
   prior <- .compact_list(utils::tail(x@numerator, 1)[[1]]@prior[[1]])
   bf_type <- .classify_BFBayesFactor(x)
 
-  prior_names <- switch(
-    bf_type,
+  prior_names <- switch(bf_type,
     "correlation" = "rho",
     "ttest1" = ,
     "ttest2" = "Difference",
@@ -309,6 +498,16 @@ get_priors.BFBayesFactor <- function(x, ...) {
     names(prior)
   )
 
+  # Distribution
+  if (bf_type == "xtable") {
+    Distribution <- x@denominator@type[[1]]
+  } else if (bf_type == "correlation") {
+    Distribution <- "beta"
+  } else {
+    Distribution <- "cauchy"
+  }
+
+  # Prior
   prior_scale <- unlist(prior)
 
   if (length(prior_names) != length(prior_scale)) {
@@ -321,12 +520,20 @@ get_priors.BFBayesFactor <- function(x, ...) {
     }))
   }
 
-  if (bf_type == "xtable") {
-    Distribution <- x@denominator@type[[1]]
-  } else {
-    Distribution <- "cauchy"
+  if (bf_type == "correlation") {
+    # "A shifted, scaled beta(1/rscale,1/rscale) prior distribution is assumed for rho"
+    prior_scale <- 1 / prior_scale
   }
 
+  # Location
+  if (bf_type == "correlation") {
+    location <- prior_scale
+  } else {
+    location <- 0
+  }
+
+
+  # Prepare output
   if (bf_type == "linear") {
     # find data types, to match priors
     data_types <- x@numerator[[1]]@dataTypes
@@ -336,7 +543,7 @@ get_priors.BFBayesFactor <- function(x, ...) {
     out <- as.data.frame(utils::stack(params), stringsAsFactors = FALSE)
     colnames(out) <- c("Parameter", "Component")
     out$Distribution <- Distribution
-    out$Location <- 0
+    out$Location <- location
     out$Scale <- NA
 
     # find parameter names pattern to match data types
@@ -357,7 +564,7 @@ get_priors.BFBayesFactor <- function(x, ...) {
     data.frame(
       Parameter = prior_names,
       Distribution = Distribution,
-      Location = 0,
+      Location = location,
       Scale = prior_scale,
       stringsAsFactors = FALSE,
       row.names = NULL
@@ -366,12 +573,16 @@ get_priors.BFBayesFactor <- function(x, ...) {
 }
 
 
+# =========================================================================
+# blavaan -------------------------------------------------------------
+# =========================================================================
+
 
 #' @export
 get_priors.blavaan <- function(x, ...) {
-  if (!requireNamespace("lavaan", quietly = TRUE)) {
-    stop("Package 'lavaan' required for this function to work. Please install it.")
-  }
+  # installed?
+  check_if_installed("lavaan")
+
 
   PE <- lavaan::parameterEstimates(
     x,
@@ -411,17 +622,14 @@ get_priors.mcmc.list <- function(x, ...) {
 }
 
 
-#' @importFrom stats na.omit
+
+
+
+# Utils -------------------------------------------------------------------
+
+
+
 .is_numeric_character <- function(x) {
-  (is.character(x) && !anyNA(suppressWarnings(as.numeric(stats::na.omit(x))))) ||
+  (is.character(x) && !anyNA(suppressWarnings(as.numeric(stats::na.omit(x[nchar(x) > 0]))))) ||
     (is.factor(x) && !anyNA(suppressWarnings(as.numeric(levels(x)))))
 }
-
-
-
-.fix_metabma_priorname <- function(x) {
-  x$Distribution <- gsub("t", "Student's t", x$Distribution, fixed = TRUE)
-  x$Distribution <- gsub("norm", "Normal", x$Distribution, fixed = TRUE)
-  x$Distribution <- gsub("invgamma", "Inverse gamma", x$Distribution, fixed = TRUE)
-  x
-}
\ No newline at end of file
