@@ -1,36 +1,38 @@
 #' @title Get variance-covariance matrix from models
 #'
 #' @description Returns the variance-covariance, as retrieved by
-#'   \code{stats::vcov()}, but works for more model objects that probably
-#'   don't provide a \code{vcov()}-method.
+#'   `stats::vcov()`, but works for more model objects that probably
+#'   don't provide a `vcov()`-method.
 #' @name get_varcov
 #'
 #' @param x A model.
 #' @param component Should the complete variance-covariance matrix of the model
 #'   be returned, or only for specific model components only (like count or
 #'   zero-inflated model parts)? Applies to models with zero-inflated component,
-#'   or models with precision (e.g. \code{betareg}) component. \code{component}
-#'   may be one of \code{"conditional"}, \code{"zi"}, \code{"zero-inflated"},
-#'   \code{"dispersion"}, \code{"precision"}, or \code{"all"}. May be abbreviated.
-#'   Note that the \emph{conditional} component is also called \emph{count} or
-#'   \emph{mean} component, depending on the model.
+#'   or models with precision (e.g. `betareg`) component. `component`
+#'   may be one of `"conditional"`, `"zi"`, `"zero-inflated"`,
+#'   `"dispersion"`, `"precision"`, or `"all"`. May be abbreviated.
+#'   Note that the *conditional* component is also called *count* or
+#'   *mean* component, depending on the model.
 #' @param effects Should the complete variance-covariance matrix of the model
 #'   be returned, or only for specific model parameters only? Currently only
-#'   applies to models of class \code{mixor}.
-#' @param complete Logical, if \code{TRUE}, for \code{aov}, returns the full
+#'   applies to models of class `mixor`.
+#' @param complete Logical, if `TRUE`, for `aov`, returns the full
 #'   variance-covariance matrix.
+#' @param robust Logical, if `TRUE`, returns a robust variance-covariance matrix
+#'   using sandwich estimation.
+#' @param verbose Toggle warnings.
 #' @param ... Currently not used.
 #'
-#' @note \code{get_varcov()} tries to return the nearest positive definite matrix
+#' @note `get_varcov()` tries to return the nearest positive definite matrix
 #'   in case of a negative variance-covariance matrix.
 #'
-#' @return The variance-covariance matrix, as \code{matrix}-object.
+#' @return The variance-covariance matrix, as `matrix`-object.
 #'
 #' @examples
 #' data(mtcars)
 #' m <- lm(mpg ~ wt + cyl + vs, data = mtcars)
 #' get_varcov(m)
-#' @importFrom stats vcov
 #' @export
 get_varcov <- function(x, ...) {
   UseMethod("get_varcov")
@@ -40,13 +42,13 @@ get_varcov <- function(x, ...) {
 # Default models ----------------------------------------------------
 
 
+#' @rdname get_varcov
 #' @export
-get_varcov.default <- function(x, ...) {
+get_varcov.default <- function(x, verbose = TRUE, ...) {
   vc <- suppressWarnings(stats::vcov(x))
-  .process_vcov(vc)
+  .process_vcov(vc, verbose)
 }
 
-
 #' @export
 get_varcov.maxLik <- get_varcov.default
 
@@ -54,8 +56,6 @@ get_varcov.maxLik <- get_varcov.default
 get_varcov.HLfit <- get_varcov.default
 
 
-
-
 # mlm ---------------------------------------------
 
 #' @export
@@ -75,22 +75,27 @@ get_varcov.mlm <- function(x, ...) {
 
 #' @rdname get_varcov
 #' @export
-get_varcov.betareg <- function(x, component = c("conditional", "precision", "all"), ...) {
+get_varcov.betareg <- function(x,
+                               component = c("conditional", "precision", "all"),
+                               verbose = TRUE,
+                               ...) {
   component <- match.arg(component)
 
-  vc <- switch(
-    component,
+  vc <- switch(component,
     "conditional" = stats::vcov(object = x, model = "mean"),
     "precision" = stats::vcov(object = x, model = "precision"),
     stats::vcov(object = x)
   )
-  .process_vcov(vc)
+  .process_vcov(vc, verbose)
 }
 
 
 #' @rdname get_varcov
 #' @export
-get_varcov.DirichletRegModel <- function(x, component = c("conditional", "precision", "all"), ...) {
+get_varcov.DirichletRegModel <- function(x,
+                                         component = c("conditional", "precision", "all"),
+                                         verbose = TRUE,
+                                         ...) {
   component <- match.arg(component)
   if (x$parametrization == "common") {
     vc <- stats::vcov(x)
@@ -107,13 +112,15 @@ get_varcov.DirichletRegModel <- function(x, component = c("conditional", "precis
       vc <- stats::vcov(x)
     }
   }
-  .process_vcov(vc)
+  .process_vcov(vc, verbose)
 }
 
 
 #' @rdname get_varcov
 #' @export
-get_varcov.clm2 <- function(x, component = c("all", "conditional", "scale"), ...) {
+get_varcov.clm2 <- function(x,
+                            component = c("all", "conditional", "scale"),
+                            ...) {
   component <- match.arg(component)
 
   n_intercepts <- length(x$xi)
@@ -126,8 +133,7 @@ get_varcov.clm2 <- function(x, component = c("all", "conditional", "scale"), ...
     vc <- .fix_negative_matrix(vc)
   }
 
-  range <- switch(
-    component,
+  range <- switch(component,
     "all" = 1:(n_scale + n_intercepts + n_location),
     "conditional" = 1:(n_intercepts + n_location),
     "scale" = (1 + n_intercepts + n_location):(n_scale + n_intercepts + n_location)
@@ -146,7 +152,9 @@ get_varcov.clmm2 <- get_varcov.clm2
 
 
 #' @export
-get_varcov.glmx <- function(x, component = c("all", "conditional", "extra"), ...) {
+get_varcov.glmx <- function(x,
+                            component = c("all", "conditional", "extra"),
+                            ...) {
   component <- match.arg(component)
   vc <- stats::vcov(object = x)
 
@@ -158,6 +166,85 @@ get_varcov.glmx <- function(x, component = c("all", "conditional", "extra"), ...
 }
 
 
+#' @export
+get_varcov.pgmm <- function(x, component = c("conditional", "all"), ...) {
+  component <- match.arg(component)
+  vc <- stats::vcov(x)
+
+  if (component != "all") {
+    keep <- match(find_parameters(x)[[component]], rownames(vc))
+    vc <- vc[keep, keep, drop = FALSE]
+  }
+  .process_vcov(vc)
+}
+
+
+#' @export
+get_varcov.selection <- function(x,
+                                 component = c("all", "selection", "outcome", "auxiliary"),
+                                 ...) {
+  component <- match.arg(component)
+  vc <- stats::vcov(object = x)
+
+  if (component != "all") {
+    keep <- match(find_parameters(x)[[component]], rownames(vc))
+    vc <- vc[keep, keep, drop = FALSE]
+  }
+
+  # we can't check for rank-deficiency here...
+  if (.is_negativ_matrix(vc)) {
+    vc <- .fix_negative_matrix(vc)
+  }
+  .remove_backticks_from_matrix_names(as.matrix(vc))
+}
+
+
+#' @export
+get_varcov.mvord <- function(x,
+                             component = c("all", "conditional", "thresholds", "correlation"),
+                             ...) {
+  component <- match.arg(component)
+  vc <- stats::vcov(x)
+
+  if (component != "all") {
+    fp <- find_parameters(x)[[component]]
+    if (component == "thresholds") {
+      fp <- gsub("\\s", "\\.", fp)
+    }
+    keep <- match(fp, rownames(vc))
+    vc <- vc[keep, keep, drop = FALSE]
+  }
+  .process_vcov(vc)
+}
+
+
+#' @export
+get_varcov.mjoint <- function(x,
+                              component = c("all", "conditional", "survival"),
+                              ...) {
+  component <- match.arg(component)
+  vc <- stats::vcov(x)
+
+  keep <- match(find_parameters(x, flatten = TRUE, component = component), rownames(vc))
+  vc <- vc[keep, keep, drop = FALSE]
+  .process_vcov(vc)
+}
+
+
+#' @export
+get_varcov.mhurdle <- function(x, component = c("all", "conditional", "zi", "zero_inflated", "infrequent_purchase", "ip", "auxiliary"), ...) {
+  component <- match.arg(component)
+  vc <- stats::vcov(x)
+
+  # rownames(vc) <- gsub("^(h1|h2|h3)\\.(.*)", "\\2", rownames(vc))
+  # colnames(vc) <- rownames(vc)
+
+  keep <- match(find_parameters(x, flatten = TRUE, component = component), rownames(vc))
+  vc <- vc[keep, keep, drop = FALSE]
+  .process_vcov(vc)
+}
+
+
 #' @rdname get_varcov
 #' @export
 get_varcov.truncreg <- function(x, component = c("conditional", "all"), ...) {
@@ -194,11 +281,12 @@ get_varcov.gamlss <- function(x, component = c("conditional", "all"), ...) {
 
 #' @rdname get_varcov
 #' @export
-get_varcov.hurdle <- function(x, component = c("conditional", "zero_inflated", "zi", "all"), ...) {
+get_varcov.hurdle <- function(x,
+                              component = c("conditional", "zero_inflated", "zi", "all"),
+                              ...) {
   component <- match.arg(component)
 
-  vc <- switch(
-    component,
+  vc <- switch(component,
     "conditional" = stats::vcov(object = x, model = "count"),
     "zi" = ,
     "zero_inflated" = stats::vcov(object = x, model = "zero"),
@@ -215,19 +303,19 @@ get_varcov.zerocount <- get_varcov.hurdle
 
 #' @rdname get_varcov
 #' @export
-get_varcov.zcpglm <- function(x, component = c("conditional", "zero_inflated", "zi", "all"), ...) {
+get_varcov.zcpglm <- function(x,
+                              component = c("conditional", "zero_inflated", "zi", "all"),
+                              ...) {
   component <- match.arg(component)
 
-  if (!requireNamespace("cplm", quietly = TRUE)) {
-    stop("To use this function, please install package 'cplm'.")
-  }
+  # installed?
+  check_if_installed("cplm")
 
   vc <- cplm::vcov(x)
   tweedie <- which(grepl("^tw_", rownames(vc)))
   zero <- which(grepl("^zero_", rownames(vc)))
 
-  vc <- switch(
-    component,
+  vc <- switch(component,
     "conditional" = vc[tweedie, tweedie, drop = FALSE],
     "zi" = ,
     "zero_inflated" = vc[zero, zero, drop = FALSE],
@@ -238,36 +326,17 @@ get_varcov.zcpglm <- function(x, component = c("conditional", "zero_inflated", "
 
 
 
-
-
-
-
 # Zero-Inflated mixed models ------------------------------------------------
 
 
 #' @rdname get_varcov
 #' @export
-get_varcov.MixMod <- function(x, component = c("conditional", "zero_inflated", "zi", "all"), ...) {
+get_varcov.glmmTMB <- function(x,
+                               component = c("conditional", "zero_inflated", "zi", "dispersion", "all"),
+                               ...) {
   component <- match.arg(component)
 
-  vc <- switch(
-    component,
-    "conditional" = stats::vcov(x, parm = "fixed-effects"),
-    "zi" = ,
-    "zero_inflated" = stats::vcov(x, parm = "zero_part"),
-    stats::vcov(x)
-  )
-  .process_vcov(vc)
-}
-
-
-#' @rdname get_varcov
-#' @export
-get_varcov.glmmTMB <- function(x, component = c("conditional", "zero_inflated", "zi", "dispersion", "all"), ...) {
-  component <- match.arg(component)
-
-  vc <- switch(
-    component,
+  vc <- switch(component,
     "conditional" = stats::vcov(x)[["cond"]],
     "zi" = ,
     "zero_inflated" = stats::vcov(x)[["zi"]],
@@ -278,6 +347,46 @@ get_varcov.glmmTMB <- function(x, component = c("conditional", "zero_inflated",
 }
 
 
+#' @rdname get_varcov
+#' @export
+get_varcov.MixMod <- function(x,
+                              effects = c("fixed", "random"),
+                              component = c("conditional", "zero_inflated", "zi", "dispersion", "auxiliary", "all"),
+                              robust = FALSE,
+                              verbose = TRUE,
+                              ...) {
+  component <- match.arg(component)
+  effects <- match.arg(effects)
+
+  random_vc <- stats::vcov(x, parm = "var-cov", sandwich = robust)
+
+  if (effects == "random") {
+    vc <- random_vc
+  } else {
+    vc <- switch(component,
+      "conditional" = stats::vcov(x, parm = "fixed-effects", sandwich = robust),
+      "zero_inflated" = ,
+      "zi" = stats::vcov(x, parm = "all", sandwich = robust),
+      "auxiliary" = ,
+      "dispersion" = stats::vcov(x, parm = "extra", sandwich = robust),
+      stats::vcov(x, parm = "all", sandwich = robust)
+    )
+
+    # drop random parameters
+    random_parms <- stats::na.omit(match(colnames(random_vc), colnames(vc)))
+    if (length(random_parms)) {
+      vc <- vc[-random_parms, -random_parms, drop = FALSE]
+    }
+
+    # filter ZI
+    if (component %in% c("zi", "zero_inflated")) {
+      zi_parms <- grepl("^zi_", colnames(vc))
+      vc <- vc[zi_parms, zi_parms, drop = FALSE]
+    }
+  }
+
+  .process_vcov(vc)
+}
 
 
 
@@ -287,8 +396,8 @@ get_varcov.glmmTMB <- function(x, component = c("conditional", "zero_inflated",
 
 #' @rdname get_varcov
 #' @export
-get_varcov.brmsfit <- function(x, component = c("conditional", "zero_inflated", "zi", "all"), ...) {
-  component <- match.arg(component)
+get_varcov.brmsfit <- function(x, component = "conditional", ...) {
+  component <- match.arg(component, choices = c("all", .all_elements()))
   params <- find_parameters(x, effects = "fixed", component = component, flatten = TRUE)
   params <- gsub("^b_", "", params)
 
@@ -297,16 +406,14 @@ get_varcov.brmsfit <- function(x, component = c("conditional", "zero_inflated",
 }
 
 
-
-
-
-
 # mfx models -------------------------------------------------------
 
 
 #' @rdname get_varcov
 #' @export
-get_varcov.betamfx <- function(x, component = c("conditional", "precision", "all"), ...) {
+get_varcov.betamfx <- function(x,
+                               component = c("conditional", "precision", "all"),
+                               ...) {
   component <- match.arg(component)
   get_varcov.betareg(x$fit, component = component, ...)
 }
@@ -337,9 +444,8 @@ get_varcov.poissonirr <- get_varcov.logitmfx
 #' @export
 get_varcov.negbinirr <- get_varcov.logitmfx
 
-
-
-
+#' @export
+get_varcov.model_fit <- get_varcov.logitmfx
 
 
 # Other models with special handling -----------------------------------------
@@ -394,11 +500,25 @@ get_varcov.robmixglm <- function(x, ...) {
 }
 
 
+#' @export
+get_varcov.Rchoice <- function(x, ...) {
+  vc <- stats::vcov(x)
+  params <- find_parameters(x, flatten = TRUE)
+  dimnames(vc) <- list(params, params)
+  .process_vcov(vc)
+}
+
+
 #' @export
 get_varcov.rq <- function(x, ...) {
   s <- summary(x, covariance = TRUE)
   vc <- as.matrix(s$cov)
 
+  # add row/column names
+  preds <- find_parameters(x, flatten = TRUE)
+  rownames(vc) <- preds
+  colnames(vc) <- preds
+
   if (.is_negativ_matrix(vc)) {
     vc <- .fix_negative_matrix(vc)
   }
@@ -407,13 +527,28 @@ get_varcov.rq <- function(x, ...) {
 }
 
 
+#' @export
+get_varcov.crr <- function(x, ...) {
+  vc <- as.matrix(x$var)
+  params <- find_parameters(x, flatten = TRUE)
+  dimnames(vc) <- list(params, params)
+  .process_vcov(vc)
+}
+
+
 #' @export
 get_varcov.crq <- function(x, ...) {
   sc <- summary(x, covariance = TRUE)
+  preds <- find_parameters(x, flatten = TRUE)
 
   if (all(unlist(lapply(sc, is.list)))) {
     vc <- lapply(sc, function(i) {
       .x <- as.matrix(i$cov)
+
+      # add row/column names
+      rownames(.x) <- preds
+      colnames(.x) <- preds
+
       if (.is_negativ_matrix(.x)) {
         .x <- .fix_negative_matrix(.x)
       }
@@ -432,10 +567,10 @@ get_varcov.crq <- function(x, ...) {
 }
 
 #' @export
-get_varcov.nlrq <- get_varcov.rq
+get_varcov.nlrq <- get_varcov.crq
 
 #' @export
-get_varcov.rqs <- get_varcov.rq
+get_varcov.rqs <- get_varcov.crq
 
 
 #' @export
@@ -448,11 +583,7 @@ get_varcov.flexsurvreg <- function(x, ...) {
 
 #' @export
 get_varcov.afex_aov <- function(x, ...) {
-  if ("lm" %in% names(x)) {
-    get_varcov(x$lm)
-  } else {
-    NULL
-  }
+  get_varcov(x$lm, ...)
 }
 
 
@@ -515,7 +646,6 @@ get_varcov.gamm <- function(x, ...) {
 }
 
 
-#' @importFrom stats cov
 #' @export
 get_varcov.lqmm <- function(x, ...) {
   sc <- summary(x, covariance = TRUE)
@@ -577,19 +707,19 @@ get_varcov.feis <- function(x, ...) {
 
 #' @export
 get_varcov.glimML <- function(x, ...) {
-  if (!requireNamespace("aod", quietly = TRUE)) {
-    stop("Package 'aod' required for this function to work. Please install it.")
-  }
+  # installed?
+  check_if_installed("aod")
+
   vc <- aod::vcov(x)
+
   .process_vcov(vc)
 }
 
 
 #' @export
 get_varcov.vglm <- function(x, ...) {
-  if (!requireNamespace("VGAM", quietly = TRUE)) {
-    stop("Package 'VGAM' required for this function to work. Please install it.")
-  }
+  # installed?
+  check_if_installed("VGAM")
   vc <- VGAM::vcov(x)
   .process_vcov(vc)
 }
@@ -645,19 +775,15 @@ get_varcov.LORgee <- get_varcov.gee
 
 
 
-
-
-
-
 # helper-functions -----------------------------------------------------
 
 
-.process_vcov <- function(vc) {
+.process_vcov <- function(vc, verbose = TRUE) {
   if (.is_negativ_matrix(vc)) {
     vc <- .fix_negative_matrix(vc)
   }
   # fix possible missings due to rank deficient model matrix
-  vc <- .fix_rank_deficiency(vc)
+  vc <- .fix_rank_deficiency(vc, verbose)
   .remove_backticks_from_matrix_names(as.matrix(vc))
 }
 
@@ -691,17 +817,28 @@ get_varcov.LORgee <- get_varcov.gee
 }
 
 
-.fix_rank_deficiency <- function(m) {
+.fix_rank_deficiency <- function(m, verbose = TRUE) {
   if (anyNA(m)) {
-    warning("Model matrix is rank deficient. Some variance-covariance parameters are missing.", call. = FALSE)
-    m <- m[!is.na]
+    if (isTRUE(verbose)) {
+      warning(format_message("Model matrix is rank deficient. Some variance-covariance parameters are missing."), call. = FALSE)
+    }
+    mm <- m[!is.na(m)]
+    if (!is.matrix(mm)) {
+      mm <- matrix(mm, nrow = sqrt(length(mm)))
+      na_cols <- apply(m, 2, function(i) all(is.na(i)))
+      rownames(mm) <- rownames(m)[!na_cols]
+      colnames(mm) <- rownames(m)[!na_cols]
+      attr(mm, "na_columns_name") <- na_cols[na_cols]
+      attr(mm, "na_columns_index") <- which(na_cols)
+      attr(mm, "rank_deficient") <- TRUE
+    }
+    m <- mm
   }
   m
 }
 
 
 
-#' @importFrom stats residuals estVar sd
 .get_weighted_varcov <- function(x, cov_unscaled) {
   ssd <- .weighted_crossprod(stats::residuals(x), w = x$weights)
   df <- sum(x$weights)
