@@ -3,9 +3,10 @@
 #'
 #' @description Returns the link-inverse function from a model object.
 #'
-#' @param what For \code{gamlss} models, indicates for which distribution
-#'   parameter the link (inverse) function should be returned; for \code{betareg}
-#'   or \code{DirichletRegModel}, can be \code{"mean"} or \code{"precision"}.
+#' @param what For `gamlss` models, indicates for which distribution
+#'   parameter the link (inverse) function should be returned; for
+#'   `betareg` or `DirichletRegModel`, can be `"mean"` or
+#'   `"precision"`.
 #' @inheritParams find_predictors
 #' @inheritParams find_formula
 #'
@@ -22,7 +23,6 @@
 #' link_inverse(m)(.3)
 #' # same as
 #' exp(.3)
-#' @importFrom stats family make.link gaussian formula
 #' @export
 link_inverse <- function(x, ...) {
   UseMethod("link_inverse")
@@ -56,14 +56,8 @@ link_inverse.default <- function(x, ...) {
 
 
 
-
-
-
-
-
 # GLM families ---------------------------------------------------
 
-
 #' @export
 link_inverse.glm <- function(x, ...) {
   tryCatch(
@@ -84,13 +78,6 @@ link_inverse.bigglm <- link_inverse.glm
 
 
 
-
-
-
-
-
-
-
 # Tobit Family ---------------------------------
 
 
@@ -115,14 +102,8 @@ link_inverse.flexsurvreg <- function(x, ...) {
 }
 
 
-
-
-
-
-
 # Gaussian identity links ---------------------------------
 
-
 #' @export
 link_inverse.lm <- function(x, ...) {
   stats::gaussian(link = "identity")$linkinv
@@ -131,6 +112,9 @@ link_inverse.lm <- function(x, ...) {
 #' @export
 link_inverse.bayesx <- link_inverse.lm
 
+#' @export
+link_inverse.systemfit <- link_inverse.lm
+
 #' @export
 link_inverse.lqmm <- link_inverse.lm
 
@@ -209,14 +193,17 @@ link_inverse.complmrob <- link_inverse.lm
 #' @export
 link_inverse.speedlm <- link_inverse.lm
 
+#' @export
+link_inverse.afex_aov <- link_inverse.lm
+
+
 
 
 #' @rdname link_inverse
 #' @export
 link_inverse.betareg <- function(x, what = c("mean", "precision"), ...) {
   what <- match.arg(what)
-  switch(
-    what,
+  switch(what,
     "mean" = x$link$mean$linkinv,
     "precision" = x$link$precision$linkinv
   )
@@ -231,8 +218,7 @@ link_inverse.DirichletRegModel <- function(x, what = c("mean", "precision"), ...
   if (x$parametrization == "common") {
     stats::make.link("logit")$linkinv
   } else {
-    switch(
-      what,
+    switch(what,
       "mean" = stats::make.link("logit")$linkinv,
       "precision" = stats::make.link("log")$linkinv
     )
@@ -241,10 +227,8 @@ link_inverse.DirichletRegModel <- function(x, what = c("mean", "precision"), ...
 
 
 
-
 # Logit links -----------------------------------
 
-
 #' @export
 link_inverse.gmnl <- function(x, ...) {
   stats::make.link("logit")$linkinv
@@ -293,26 +277,24 @@ link_inverse.logistf <- link_inverse.gmnl
 link_inverse.multinom <- link_inverse.gmnl
 
 
-
-
-
-
-
 # Probit link ------------------------
 
-
 #' @export
 link_inverse.ivprobit <- function(x, ...) {
   stats::make.link(link = "probit")$linkinv
 }
 
 
-
+#' @export
+link_inverse.mvord <- function(x, ...) {
+  link_name <- x$rho$link$name
+  l <- stats::make.link(link = ifelse(link_name == "mvprobit", "probit", "logit"))
+  l$linkinv
+}
 
 
 # Log-links ---------------------------------------
 
-
 #' @export
 link_inverse.zeroinfl <- function(x, ...) {
   stats::make.link("log")$linkinv
@@ -326,15 +308,8 @@ link_inverse.zerotrunc <- link_inverse.zeroinfl
 
 
 
-
-
-
-
-
-
 # Ordinal models -----------------------------------
 
-
 #' @export
 link_inverse.clm <- function(x, ...) {
   stats::make.link(.get_ordinal_link(x))$linkinv
@@ -351,10 +326,6 @@ link_inverse.mixor <- link_inverse.clm
 
 
 
-
-
-
-
 # mfx models ------------------------------------------------------
 
 
@@ -393,13 +364,13 @@ link_inverse.negbinirr <- link_inverse.logitmfx
 
 
 
-
-
-
-
-
 # Other models ----------------------------
 
+#' @export
+link_inverse.Rchoice <- function(x, ...) {
+  stats::make.link(link = x$link)$linkinv
+}
+
 
 #' @export
 link_inverse.merModList <- function(x, ...) {
@@ -409,8 +380,7 @@ link_inverse.merModList <- function(x, ...) {
 
 #' @export
 link_inverse.robmixglm <- function(x, ...) {
-  switch(
-    tolower(x$family),
+  switch(tolower(x$family),
     gaussian = stats::make.link(link = "identity")$linkinv,
     binomial = stats::make.link(link = "logit")$linkinv,
     gamma = stats::make.link(link = "inverse")$linkinv,
@@ -433,7 +403,6 @@ link_inverse.cglm <- function(x, ...) {
 }
 
 
-
 #' @export
 link_inverse.cpglmm <- function(x, ...) {
   f <- .get_cplm_family(x)
@@ -459,8 +428,7 @@ link_inverse.fixest <- function(x, ...) {
   } else if (inherits(x$family, "family")) {
     x$family$linkinv
   } else {
-    link <- switch(
-      x$family,
+    link <- switch(x$family,
       "poisson" = ,
       "negbin" = "log",
       "logit" = "logit",
@@ -548,8 +516,7 @@ link_inverse.glmmTMB <- function(x, ...) {
 
 #' @export
 link_inverse.MCMCglmm <- function(x, ...) {
-  switch(
-    x$Residual$original.family,
+  switch(x$Residual$original.family,
     "cengaussian" = ,
     "gaussian" = stats::gaussian(link = "identity")$linkinv,
     "categorical" = ,
@@ -568,8 +535,7 @@ link_inverse.MCMCglmm <- function(x, ...) {
 
 #' @export
 link_inverse.glmm <- function(x, ...) {
-  switch(
-    tolower(x$family.glmm$family.glmm),
+  switch(tolower(x$family.glmm$family.glmm),
     "bernoulli.glmm" = ,
     "binomial.glmm" = stats::make.link("logit")$linkinv,
     "poisson.glmm" = stats::make.link("log")$linkinv,
@@ -595,8 +561,7 @@ link_inverse.stanmvreg <- function(x, ...) {
 
 #' @export
 link_inverse.gbm <- function(x, ...) {
-  switch(
-    x$distribution$name,
+  switch(x$distribution$name,
     laplace = ,
     tdist = ,
     gaussian = stats::gaussian(link = "identity")$linkinv,
@@ -625,8 +590,7 @@ link_inverse.brmsfit <- function(x, ...) {
 link_inverse.gamlss <- function(x, what = c("mu", "sigma", "nu", "tau"), ...) {
   what <- match.arg(what)
   faminfo <- get(x$family[1], asNamespace("gamlss"))()
-  switch(
-    what,
+  switch(what,
     "mu" = faminfo$mu.linkinv,
     "sigma" = faminfo$sigma.linkinv,
     "nu" = faminfo$nu.linkinv,
@@ -662,8 +626,6 @@ link_inverse.MixMod <- link_inverse.glmmPQL
 link_inverse.cgam <- link_inverse.glmmPQL
 
 
-
-
 #' @export
 link_inverse.vgam <- function(x, ...) {
   x@family@linkinv
@@ -672,6 +634,14 @@ link_inverse.vgam <- function(x, ...) {
 #' @export
 link_inverse.vglm <- link_inverse.vgam
 
+#' @export
+link_inverse.svy_vglm <- function(x, ...) {
+  link_inverse(x$fit)
+}
+
+#' @export
+link_inverse.model_fit <- link_inverse.svy_vglm
+
 
 #' @export
 link_inverse.gam <- function(x, ...) {
@@ -708,19 +678,15 @@ link_inverse.mipo <- function(x, ...) {
 
 #' @export
 link_inverse.mira <- function(x, ...) {
-  if (!requireNamespace("mice", quietly = TRUE)) {
-    stop("Package `mice` required. Please install it.", call. = FALSE)
-  }
+  # installed?
+  check_if_installed("mice")
   link_inverse(mice::pool(x), ...)
 }
 
 
 
-
-
 # helper --------------
 
-
 .brms_link_inverse <- function(fam) {
   # do we have custom families?
   if (!is.null(fam$family) && (is.character(fam$family) && fam$family == "custom")) {
@@ -740,7 +706,6 @@ link_inverse.mira <- function(x, ...) {
 }
 
 
-#' @importFrom stats poisson
 .get_cplm_family <- function(x) {
   link <- parse(text = .safe_deparse(x@call))[[1]]$link
 
@@ -749,9 +714,8 @@ link_inverse.mira <- function(x, ...) {
   if (!is.numeric(link)) {
     stats::poisson(link = link)
   } else {
-    if (!requireNamespace("statmod", quietly = TRUE)) {
-      stop("Package 'statmod' required. Please install it.")
-    }
+    # installed?
+    check_if_installed("statmod")
     statmod::tweedie(link.power = link)
   }
 }
