@@ -11,50 +11,51 @@
 #' @return A list with information about the model, like family, link-function
 #'   etc. (see 'Details').
 #'
-#' @details \code{model_info()} returns a list with information about the
-#'   model for many different model objects. Following information
-#'    is returned, where all values starting with \code{is_} are logicals.
-#'    \itemize{
-#'      \item \code{is_binomial}: family is binomial (but not negative binomial)
-#'      \item \code{is_poisson}: family is poisson
-#'      \item \code{is_negbin}: family is negative binomial
-#'      \item \code{is_count}: model is a count model (i.e. family is either poisson or negative binomial)
-#'      \item \code{is_beta}: family is beta
-#'      \item \code{is_betabinomial}: family is beta-binomial
-#'      \item \code{is_dirichlet}: family is dirichlet
-#'      \item \code{is_exponential}: family is exponential (e.g. Gamma or Weibull)
-#'      \item \code{is_logit}: model has logit link
-#'      \item \code{is_probit}: model has probit link
-#'      \item \code{is_linear}: family is gaussian
-#'      \item \code{is_tweedie}: family is tweedie
-#'      \item \code{is_ordinal}: family is ordinal or cumulative link
-#'      \item \code{is_cumulative}: family is ordinal or cumulative link
-#'      \item \code{is_multinomial}: family is multinomial or categorical link
-#'      \item \code{is_categorical}: family is categorical link
-#'      \item \code{is_censored}: model is a censored model (has a censored response, including survival models)
-#'      \item \code{is_truncated}: model is a truncated model (has a truncated response)
-#'      \item \code{is_survival}: model is a survival model
-#'      \item \code{is_zero_inflated}: model has zero-inflation component
-#'      \item \code{is_hurdle}: model has zero-inflation component and is a hurdle-model (truncated family distribution)
-#'      \item \code{is_dispersion}: model has dispersion component
-#'      \item \code{is_mixed}: model is a mixed effects model (with random effects)
-#'      \item \code{is_multivariate}: model is a multivariate response model (currently only works for \emph{brmsfit} objects)
-#'      \item \code{is_trial}: model response contains additional information about the trials
-#'      \item \code{is_bayesian}: model is a Bayesian model
-#'      \item \code{is_anova}: model is an Anova object
-#'      \item \code{is_ttest}: model is an an object of class \code{htest}, returned by \code{t.test()}
-#'      \item \code{is_correlation}: model is an an object of class \code{htest}, returned by \code{cor.test()}
-#'      \item \code{is_ranktest}: model is an an object of class \code{htest}, returned by \code{cor.test()} (if Spearman's rank correlation), \code{wilcox.text()} or \code{kruskal.test()}.
-#'      \item \code{is_onewaytest}: model is an an object of class \code{htest}, returned by \code{oneway.test()}
-#'      \item \code{is_proptest}: model is an an object of class \code{htest}, returned by \code{prop.test()}
-#'      \item \code{is_binomtest}: model is an an object of class \code{htest}, returned by \code{binom.test()}
-#'      \item \code{is_chi2test}: model is an an object of class \code{htest}, returned by \code{chisq.test()}
-#'      \item \code{is_xtab}: model is an an object of class \code{htest} or \code{BFBayesFactor}, and test-statistic stems from a contingency table (i.e. \code{chisq.test()} or \code{BayesFactor::contingencyTableBF()}).
-#'      \item \code{link_function}: the link-function
-#'      \item \code{family}: the family-object
-#'      \item \code{n_obs}: number of observations
-#'      \item \code{model_terms}: a list with all model terms, including terms such as random effects or from zero-inflated model parts.
-#'    }
+#' @details `model_info()` returns a list with information about the
+#' model for many different model objects. Following information
+#' is returned, where all values starting with `is_` are logicals.
+#' * `is_binomial`: family is binomial (but not negative binomial)
+#' * `is_bernoulli`: special case of binomial models: family is Bernoulli
+#' * `is_poisson`: family is poisson
+#' * `is_negbin`: family is negative binomial
+#' * `is_count`: model is a count model (i.e. family is either poisson or negative binomial)
+#' * `is_beta`: family is beta
+#' * `is_betabinomial`: family is beta-binomial
+#' * `is_dirichlet`: family is dirichlet
+#' * `is_exponential`: family is exponential (e.g. Gamma or Weibull)
+#' * `is_logit`: model has logit link
+#' * `is_probit`: model has probit link
+#' * `is_linear`: family is gaussian
+#' * `is_tweedie`: family is tweedie
+#' * `is_ordinal`: family is ordinal or cumulative link
+#' * `is_cumulative`: family is ordinal or cumulative link
+#' * `is_multinomial`: family is multinomial or categorical link
+#' * `is_categorical`: family is categorical link
+#' * `is_censored`: model is a censored model (has a censored response, including survival models)
+#' * `is_truncated`: model is a truncated model (has a truncated response)
+#' * `is_survival`: model is a survival model
+#' * `is_zero_inflated`: model has zero-inflation component
+#' * `is_hurdle`: model has zero-inflation component and is a hurdle-model (truncated family distribution)
+#' * `is_dispersion`: model has dispersion component
+#' * `is_mixed`: model is a mixed effects model (with random effects)
+#' * `is_multivariate`: model is a multivariate response model (currently only works for _brmsfit_ objects)
+#' * `is_trial`: model response contains additional information about the trials
+#' * `is_bayesian`: model is a Bayesian model
+#' * `is_gam`: model is a generalized additive model
+#' * `is_anova`: model is an Anova object
+#' * `is_ttest`: model is an an object of class `htest`, returned by `t.test()`
+#' * `is_correlation`: model is an an object of class `htest`, returned by `cor.test()`
+#' * `is_ranktest`: model is an an object of class `htest`, returned by `cor.test()` (if Spearman's rank correlation), `wilcox.text()` or `kruskal.test()`.
+#' * `is_levenetest`: model is an an object of class `anova`, returned by `car::leveneTest()`.
+#' * `is_onewaytest`: model is an an object of class `htest`, returned by `oneway.test()`
+#' * `is_proptest`: model is an an object of class `htest`, returned by `prop.test()`
+#' * `is_binomtest`: model is an an object of class `htest`, returned by `binom.test()`
+#' * `is_chi2test`: model is an an object of class `htest`, returned by `chisq.test()`
+#' * `is_xtab`: model is an an object of class `htest` or `BFBayesFactor`, and test-statistic stems from a contingency table (i.e. `chisq.test()` or `BayesFactor::contingencyTableBF()`).
+#' * `link_function`: the link-function
+#' * `family`: the family-object
+#' * `n_obs`: number of observations
+#' * `model_terms`: a list with all model terms, including terms such as random effects or from zero-inflated model parts.
 #'
 #' @examples
 #' ldose <- rep(0:5, 2)
@@ -78,25 +79,20 @@
 #' }
 #'
 #' model_info(m)
-#' @importFrom stats formula terms
 #' @export
 model_info <- function(x, ...) {
   UseMethod("model_info")
 }
 
 
-
-
-
 # Default methods --------------------------------------
 
 #' @export
 model_info.data.frame <- function(x, ...) {
-  stop("A data frame is no valid object for this function")
+  stop("A data frame is no valid object for this function.", call. = FALSE)
 }
 
 
-#' @importFrom stats family
 #' @rdname model_info
 #' @export
 model_info.default <- function(x, verbose = TRUE, ...) {
@@ -124,6 +120,7 @@ model_info.default <- function(x, verbose = TRUE, ...) {
       fitfam = faminfo$family,
       logit.link = faminfo$link == "logit",
       link.fun = faminfo$link,
+      verbose = verbose,
       ...
     )
   } else {
@@ -135,20 +132,36 @@ model_info.default <- function(x, verbose = TRUE, ...) {
 }
 
 
-
+#' @export
+model_info.model_fit <- function(x, verbose = TRUE, ...) {
+  model_info(x$fit, verbose = verbose, ...)
+}
 
 
 # Models with general handling, Gaussian ----------------------------------
 
 
 #' @export
-model_info.mmclogit <- function(x, ...) {
-  .make_family(x, ...)
+model_info.anova <- function(x, verbose = TRUE, ...) {
+  if (!is.null(attributes(x)$heading) && grepl("Levene's Test", attributes(x)$heading, fixed = TRUE)) {
+    .make_family(x, verbose = verbose)
+  } else {
+    NULL
+  }
+}
+
+
+#' @export
+model_info.mmclogit <- function(x, verbose = TRUE, ...) {
+  .make_family(x, verbose = verbose, ...)
 }
 
 #' @export
 model_info.maxLik <- model_info.mmclogit
 
+#' @export
+model_info.mjoint <- model_info.mmclogit
+
 #' @export
 model_info.censReg <- model_info.mmclogit
 
@@ -224,6 +237,9 @@ model_info.lm_robust <- model_info.mmclogit
 #' @export
 model_info.iv_robust <- model_info.mmclogit
 
+#' @export
+model_info.systemfit <- model_info.mmclogit
+
 #' @export
 model_info.lqmm <- model_info.mmclogit
 
@@ -269,30 +285,26 @@ model_info.mlm <- function(x, ...) {
 }
 
 #' @export
-model_info.afex_aov <- function(x, ...) {
-  if ("aov" %in% names(x)) {
-    .make_family(x$aov, ...)
+model_info.afex_aov <- function(x, verbose = TRUE, ...) {
+  if (!is.null(x$aov)) {
+    .make_family(x$aov, verbose = verbose, ...)
   } else {
-    .make_family(x$lm, ...)
+    .make_family(x$lm, verbose = verbose, ...)
   }
 }
 
 
-
-
-
-
 # Models with logit-link --------------------------------
 
-
 #' @export
-model_info.logistf <- function(x, ...) {
+model_info.logistf <- function(x, verbose = TRUE, ...) {
   faminfo <- stats::binomial(link = "logit")
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
@@ -300,6 +312,9 @@ model_info.logistf <- function(x, ...) {
 #' @export
 model_info.lrm <- model_info.logistf
 
+#' @export
+model_info.blrm <- model_info.logistf
+
 #' @export
 model_info.multinom <- model_info.logistf
 
@@ -311,22 +326,18 @@ model_info.gmnl <- model_info.logistf
 
 
 
-
-
-
-
-
 # Models with ordinal family ------------------------------------
 
 
 #' @export
-model_info.clm <- function(x, ...) {
+model_info.clm <- function(x, verbose = TRUE, ...) {
   faminfo <- stats::binomial(link = .get_ordinal_link(x))
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
@@ -340,38 +351,48 @@ model_info.clmm <- model_info.clm
 #' @export
 model_info.mixor <- model_info.clm
 
-
-
-
-
+#' @export
+model_info.mvord <- function(x, verbose = verbose, ...) {
+  link_name <- x$rho$link$name
+  faminfo <- stats::binomial(link = ifelse(link_name == "mvprobit", "probit", "logit"))
+  .make_family(
+    x = x,
+    fitfam = faminfo$family,
+    logit.link = faminfo$link == "logit",
+    link.fun = faminfo$link,
+    multi.var = TRUE,
+    verbose = verbose,
+    ...
+  )
+}
 
 
 
 # Models with family-function  ----------------------------------
 
-
-#' @importFrom stats family
 #' @export
-model_info.bamlss <- function(x, ...) {
+model_info.bamlss <- function(x, verbose = TRUE, ...) {
   faminfo <- stats::family(x)
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$links[1] == "logit",
     link.fun = faminfo$links[1],
+    verbose = verbose,
     ...
   )
 }
 
 
 #' @export
-model_info.speedglm <- function(x, ...) {
+model_info.speedglm <- function(x, verbose = TRUE, ...) {
   faminfo <- stats::family(x)
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
@@ -381,15 +402,10 @@ model_info.brmultinom <- model_info.speedglm
 
 
 
-
-
-
-
 # Models with tobit family ----------------------------------
 
-
 #' @export
-model_info.flexsurvreg <- function(x, ...) {
+model_info.flexsurvreg <- function(x, verbose = TRUE, ...) {
   dist <- parse(text = .safe_deparse(x$call))[[1]]$dist
   faminfo <- .make_tobit_family(x, dist)
 
@@ -398,12 +414,13 @@ model_info.flexsurvreg <- function(x, ...) {
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
 
 #' @export
-model_info.tobit <- function(x, ...) {
+model_info.tobit <- function(x, verbose = TRUE, ...) {
   faminfo <- .make_tobit_family(x)
 
   .make_family(
@@ -411,6 +428,7 @@ model_info.tobit <- function(x, ...) {
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
@@ -423,21 +441,18 @@ model_info.survreg <- model_info.tobit
 
 
 
-
-
-
-
 # Models with family in object ----------------------------------
 
 
 #' @export
-model_info.MixMod <- function(x, ...) {
+model_info.MixMod <- function(x, verbose = TRUE, ...) {
   faminfo <- x$family
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
@@ -450,20 +465,21 @@ model_info.bife <- model_info.MixMod
 
 
 #' @export
-model_info.glmx <- function(x, ...) {
+model_info.glmx <- function(x, verbose = TRUE, ...) {
   faminfo <- x$family$glm
   .make_family(
     x = x,
     fitfam = faminfo$family,
     logit.link = faminfo$link == "logit",
     link.fun = faminfo$link,
+    verbose = verbose,
     ...
   )
 }
 
 
 #' @export
-model_info.fixest <- function(x, ...) {
+model_info.fixest <- function(x, verbose = TRUE, ...) {
   faminfo <- x$family
 
   if (is.null(faminfo)) {
@@ -476,18 +492,17 @@ model_info.fixest <- function(x, ...) {
       fitfam = faminfo$family,
       logit.link = faminfo$link == "logit",
       link.fun = faminfo$link,
+      verbose = verbose,
       ...
     )
   } else {
-    fitfam <- switch(
-      faminfo,
+    fitfam <- switch(faminfo,
       "negbin" = "negative binomial",
       "logit" = "binomial",
       faminfo
     )
 
-    link <- switch(
-      faminfo,
+    link <- switch(faminfo,
       "poisson" = ,
       "negbin" = "log",
       "logit" = "logit",
@@ -499,6 +514,7 @@ model_info.fixest <- function(x, ...) {
       fitfam = fitfam,
       logit.link = link == "logit",
       link.fun = link,
+      verbose = verbose,
       ...
     )
   }
@@ -508,21 +524,16 @@ model_info.fixest <- function(x, ...) {
 model_info.feglm <- model_info.fixest
 
 
-
-
-
-
-
 # Survival-models ----------------------------------------
 
-
 #' @export
-model_info.coxph <- function(x, ...) {
+model_info.coxph <- function(x, verbose = TRUE, ...) {
   .make_family(
     x = x,
     fitfam = "survival",
     logit.link = TRUE,
     link.fun = NULL,
+    verbose = verbose,
     ...
   )
 }
@@ -547,13 +558,8 @@ model_info.comprisk <- model_info.coxph
 
 
 
-
-
-
-
 # Zero-Inflated Models ------------------------------
 
-
 #' @export
 model_info.zeroinfl <- function(x, ...) {
   if (is.list(x$dist)) {
@@ -561,8 +567,7 @@ model_info.zeroinfl <- function(x, ...) {
   } else {
     dist <- x$dist
   }
-  fitfam <- switch(
-    dist,
+  fitfam <- switch(dist,
     poisson = "poisson",
     negbin = "negative binomial",
     "poisson"
@@ -587,8 +592,7 @@ model_info.hurdle <- function(x, ...) {
   } else {
     dist <- x$dist
   }
-  fitfam <- switch(
-    dist,
+  fitfam <- switch(dist,
     poisson = "poisson",
     negbin = "negative binomial",
     "poisson"
@@ -604,11 +608,30 @@ model_info.hurdle <- function(x, ...) {
 }
 
 
+#' @export
+model_info.mhurdle <- function(x, ...) {
+  .make_family(
+    x = x,
+    zero.inf = TRUE,
+    hurdle = TRUE,
+    ...
+  )
+}
 
 
 # Bayesian Models ---------------------------
 
 
+#' @export
+model_info.stanreg <- function(x, ...) {
+  if (inherits(x, "polr")) {
+    model_info.polr(x)
+  } else {
+    model_info.default(x)
+  }
+}
+
+
 #' @export
 model_info.brmsfit <- function(x, ...) {
   faminfo <- stats::family(x)
@@ -621,6 +644,7 @@ model_info.brmsfit <- function(x, ...) {
         logit.link = .x$link == "logit",
         multi.var = TRUE,
         link.fun = .x$link,
+        dispersion = !.is_empty_object(insight::find_formula(x)$sigma),
         ...
       )
     })
@@ -631,6 +655,7 @@ model_info.brmsfit <- function(x, ...) {
       logit.link = faminfo$link == "logit",
       multi.var = FALSE,
       link.fun = faminfo$link,
+      dispersion = !.is_empty_object(insight::find_formula(x)$sigma),
       ...
     )
   }
@@ -656,14 +681,12 @@ model_info.stanmvreg <- function(x, ...) {
 
 #' @export
 model_info.BGGM <- function(x, ...) {
-  link <- switch(
-    x$type,
+  link <- switch(x$type,
     "continuous" = stats::gaussian(),
     stats::binomial()
   )
 
-  family <- switch(
-    x$type,
+  family <- switch(x$type,
     "continuous" = "gaussian",
     "binary" = "binomial",
     "ordinal"
@@ -680,13 +703,28 @@ model_info.BGGM <- function(x, ...) {
 }
 
 
-
-
-
-
-
 # Other models ----------------------------
 
+#' @export
+model_info.garch <- function(x, ...) {
+  .make_family(
+    x = x,
+    ...
+  )
+}
+
+
+#' @export
+model_info.Rchoice <- function(x, ...) {
+  .make_family(
+    x = x,
+    fitfam = x$family,
+    logit.link = x$link == "logit",
+    link.fun = x$link,
+    ...
+  )
+}
+
 
 #' @export
 model_info.ivprobit <- function(x, ...) {
@@ -701,8 +739,8 @@ model_info.ivprobit <- function(x, ...) {
 
 
 #' @export
-model_info.glht <- function(x, ...) {
-  model_info(x$model, ...)
+model_info.glht <- function(x, verbose = TRUE, ...) {
+  model_info(x$model, verbose = verbose, ...)
 }
 
 
@@ -714,8 +752,7 @@ model_info.coeftest <- function(x, ...) {
 
 #' @export
 model_info.glmm <- function(x, ...) {
-  f <- switch(
-    tolower(x$family.glmm$family.glmm),
+  f <- switch(tolower(x$family.glmm$family.glmm),
     "bernoulli.glmm" = ,
     "binomial.glmm" = stats::binomial("logit"),
     "poisson.glmm" = stats::poisson("log"),
@@ -734,8 +771,7 @@ model_info.glmm <- function(x, ...) {
 
 #' @export
 model_info.robmixglm <- function(x, ...) {
-  f <- switch(
-    tolower(x$family),
+  f <- switch(tolower(x$family),
     gaussian = stats::gaussian("identity"),
     binomial = stats::binomial("logit"),
     poisson = stats::poisson("log"),
@@ -927,7 +963,10 @@ model_info.bcplm <- model_info.cpglmm
 
 #' @export
 model_info.glimML <- function(x, ...) {
-  fitfam <- switch(x@method, BB = "betabinomial", NB = "negative binomial")
+  fitfam <- switch(x@method,
+    BB = "betabinomial",
+    NB = "negative binomial"
+  )
   .make_family(
     x = x,
     fitfam = fitfam,
@@ -957,7 +996,7 @@ model_info.gam <- function(x, ...) {
   .make_family(
     x = x,
     fitfam = faminfo$family,
-    logit.link = link == "logit",
+    logit.link = !is.null(link) && (link == "logit" || faminfo$family == "multinom"),
     link.fun = link,
     multi.var = is.mv,
     ...
@@ -984,12 +1023,16 @@ model_info.vgam <- function(x, ...) {
 model_info.vglm <- model_info.vgam
 
 
+#' @export
+model_info.svy_vglm <- function(x, verbose = TRUE, ...) {
+  model_info(x$fit, verbose = verbose)
+}
+
 
 #' @export
 model_info.glmmTMB <- function(x, ...) {
-  if (!requireNamespace("lme4", quietly = TRUE)) {
-    stop("To use this function, please install package 'lme4'.")
-  }
+  # installed?
+  check_if_installed("lme4")
 
   faminfo <- stats::family(x)
   .make_family(
@@ -1034,8 +1077,7 @@ model_info.DirichletRegModel <- function(x, ...) {
 
 #' @export
 model_info.gbm <- function(x, ...) {
-  faminfo <- switch(
-    x$distribution$name,
+  faminfo <- switch(x$distribution$name,
     laplace = ,
     tdist = ,
     gaussian = list(name = "gaussian", logit = FALSE, link = NULL),
@@ -1102,7 +1144,10 @@ model_info.orm <- function(x, ...) {
 
 #' @export
 model_info.svyolr <- function(x, ...) {
-  l <- switch(x$method, logistic = "logit", x$method)
+  l <- switch(x$method,
+    logistic = "logit",
+    x$method
+  )
   faminfo <- stats::binomial(link = l)
   .make_family(
     x = x,
@@ -1129,11 +1174,11 @@ model_info.gamlss <- function(x, ...) {
 
 
 #' @export
-model_info.mipo <- function(x, ...) {
+model_info.mipo <- function(x, verbose = TRUE, ...) {
   tryCatch(
     {
       models <- eval(x$call$object)
-      model_info(models$analyses[[1]], ...)
+      model_info(models$analyses[[1]], verbose = verbose, ...)
     },
     error = function(e) {
       NULL
@@ -1148,12 +1193,8 @@ model_info.mira <- function(x, ...) {
 }
 
 
-
-
-
 # mfx models -------------------------------
 
-
 #' @export
 model_info.betamfx <- function(x, ...) {
   model_info.betareg(x$fit)
@@ -1184,3 +1225,26 @@ model_info.poissonirr <- model_info.logitmfx
 
 #' @export
 model_info.negbinirr <- model_info.logitmfx
+
+
+#' @export
+model_info.bfsl <- function(x, verbose = TRUE, ...) {
+  .make_family(
+    x = x,
+    fitfam = "gaussian",
+    logit.link = FALSE,
+    link.fun = "identity",
+    verbose = verbose,
+    ...
+  )
+}
+
+
+
+
+# not yet supported -------------------------------
+
+#' @export
+model_info.earth <- function(x, ...) {
+  stop("Models of class 'earth' are not yet supported.", call. = FALSE)
+}
