@@ -1,4 +1,18 @@
-.make_family <- function(x, fitfam = "gaussian", zero.inf = FALSE, hurdle = FALSE, logit.link = FALSE, multi.var = FALSE, link.fun = "identity", dispersion = FALSE, ...) {
+.make_family <- function(x,
+                         fitfam = "gaussian",
+                         zero.inf = FALSE,
+                         hurdle = FALSE,
+                         logit.link = FALSE,
+                         multi.var = FALSE,
+                         link.fun = "identity",
+                         dispersion = FALSE,
+                         verbose = TRUE,
+                         ...) {
+  dots <- list(...)
+  if (isTRUE(dots$return_family_only)) {
+    return(list(family = fitfam, link_function = link.fun))
+  }
+
   # create logical for family
 
   # binomial family --------
@@ -28,6 +42,23 @@
       fitfam %in% c("ztnbinom", "nbinom")
 
 
+  # bernoulli family --------
+
+  is_bernoulli <- FALSE
+
+  if (binom_fam && inherits(x, "glm") && !neg_bin_fam && !poisson_fam) {
+    resp <- get_response(x, verbose = FALSE)
+    if (is.data.frame(resp) && ncol(resp) == 1) {
+      resp <- as.vector(resp[[1]])
+    }
+    if (!is.data.frame(resp) && all(.is.int(.factor_to_numeric(resp[[1]])))) {
+      is_bernoulli <- TRUE
+    }
+  } else if (fitfam %in% "bernoulli") {
+    is_bernoulli <- TRUE
+  }
+
+
   # beta family --------
 
   beta_fam <-
@@ -81,7 +112,7 @@
   # ordinal family --------
 
   is.ordinal <-
-    inherits(x, c("svyolr", "polr", "clm", "clm2", "clmm", "mixor", "LORgee")) |
+    inherits(x, c("svyolr", "polr", "clm", "clm2", "clmm", "mixor", "LORgee", "mvord")) |
       fitfam %in% c("cumulative", "ordinal")
 
 
@@ -89,7 +120,7 @@
 
   is.multinomial <-
     inherits(x, c("gmnl", "mlogit", "DirichletRegModel", "multinom", "brmultinom")) |
-      fitfam %in% c("cratio", "sratio", "acat", "multinomial", "multinomial2", "dirichlet")
+      fitfam %in% c("cratio", "sratio", "acat", "multinom", "multinomial", "multinomial2", "dirichlet")
 
 
   # categorical family --------
@@ -97,19 +128,40 @@
   is.categorical <- fitfam == "categorical"
 
 
+  # special handling of rms --------------
+
+  if (inherits(x, c("lrm", "blrm"))) {
+    resp <- get_response(x, verbose = FALSE)
+    if (.n_unique(resp) == 2) {
+      binom_fam <- TRUE
+    } else {
+      is.ordinal <- TRUE
+    }
+  }
+
+
   # Bayesian model --------
 
-  is.bayes <- inherits(x, c(
-    "brmsfit", "stanfit", "MCMCglmm", "stanreg",
-    "stanmvreg", "bmerMod", "BFBayesFactor", "bamlss",
-    "bayesx", "mcmc", "bcplm", "bayesQR", "BGGM",
-    "meta_random", "meta_fixed", "meta_bma", "blavaan"
-  ))
+  is.bayes <- .is_bayesian_model(x)
 
 
   # survival model --------
 
-  is.survival <- inherits(x, c("aareg", "survreg", "survfit", "survPresmooth", "flexsurvreg", "coxph", "coxme", "coxr", "riskRegression", "comprisk"))
+  is.survival <- inherits(
+    x,
+    c(
+      "aareg",
+      "survreg",
+      "survfit",
+      "survPresmooth",
+      "flexsurvreg",
+      "coxph",
+      "coxme",
+      "coxr",
+      "riskRegression",
+      "comprisk"
+    )
+  )
 
   # check if we have binomial models with trials instead of binary outcome
   # and check if we have truncated or censored brms-regression
@@ -153,8 +205,7 @@
 
   # save model terms --------
 
-  dots <- list(...)
-  if (.obj_has_name(dots, "no_terms") && isTRUE(dots$no_terms)) {
+  if (isTRUE(dots$no_terms)) {
     model_terms <- NULL
   } else {
     if (inherits(x, "mcmc")) {
@@ -162,7 +213,13 @@
     } else {
       model_terms <- tryCatch(
         {
-          find_variables(x, effects = "all", component = "all", flatten = FALSE)
+          find_variables(
+            x,
+            effects = "all",
+            component = "all",
+            flatten = FALSE,
+            verbose = FALSE
+          )
         },
         error = function(x) {
           NULL
@@ -182,9 +239,15 @@
   is_chi2test <- FALSE
   is_ranktest <- FALSE
   is_xtab <- FALSE
+  is_levenetest <- FALSE
 
   if (inherits(x, "htest")) {
-    if (grepl("kruskal-wallis", tolower(x$method), fixed = TRUE) || grepl("wilcoxon", tolower(x$method), fixed = TRUE)) {
+    if (grepl("kruskal-wallis", tolower(x$method), fixed = TRUE) ||
+      grepl("design-based kruskalwallis", tolower(x$method), fixed = TRUE) ||
+      grepl("design-based median", tolower(x$method), fixed = TRUE) ||
+      grepl("design-based vanderwaerden", tolower(x$method), fixed = TRUE) ||
+      grepl("wilcoxon", tolower(x$method), fixed = TRUE) ||
+      grepl("friedman", tolower(x$method), fixed = TRUE)) {
       is_ranktest <- TRUE
     } else if (grepl("t-test", x$method)) {
       is_ttest <- TRUE
@@ -199,7 +262,8 @@
       is_proptest <- TRUE
       fitfam <- "binomial"
     } else if (any(grepl("chi-squared", c(tolower(x$method), tolower(attributes(x$statistic)$names)), fixed = TRUE)) ||
-               grepl("Fisher's Exact Test", x$method, fixed = TRUE)) {
+      grepl("Fisher's Exact Test", x$method, fixed = TRUE) ||
+      grepl("pearson's x^2", tolower(x$method), fixed = TRUE)) {
       is_chi2test <- TRUE
       is_xtab <- TRUE
       fitfam <- "categorical"
@@ -213,6 +277,10 @@
     is_correlation <- TRUE
   }
 
+  # exceptions: car::leveneTest
+  if (inherits(x, "anova") && !is.null(attributes(x)$heading) && grepl("Levene's Test", attributes(x)$heading, fixed = TRUE)) {
+    is_levenetest <- TRUE
+  }
 
   # Bayesfactors terms --------
 
@@ -250,7 +318,7 @@
   }
 
   if (inherits(x, "brmsfit") && !is_multivariate(x)) {
-    is_meta <- grepl("(.*)\\|(.*)se\\((.*)\\)", .safe_deparse(find_formula(x)$conditional[[2]]))
+    is_meta <- grepl("(.*)\\|(.*)se\\((.*)\\)", .safe_deparse(find_formula(x, verbose = FALSE)$conditional[[2]]))
   }
 
 
@@ -279,6 +347,7 @@
 
   list(
     is_binomial = binom_fam & !neg_bin_fam,
+    is_bernoulli = is_bernoulli,
     is_count = poisson_fam | neg_bin_fam,
     is_poisson = poisson_fam,
     is_negbin = neg_bin_fam,
@@ -301,10 +370,11 @@
     is_cumulative = is.ordinal,
     is_multinomial = is.multinomial | is.categorical,
     is_categorical = is.categorical,
-    is_mixed = is_mixed_model(x),
+    is_mixed = !is_levenetest && is_mixed_model(x),
     is_multivariate = multi.var,
     is_trial = is.trial,
     is_bayesian = is.bayes,
+    is_gam = is_gam_model(x),
     is_anova = inherits(x, c("aov", "aovlist", "MANOVA", "RM")),
     is_timeseries = inherits(x, c("Arima")),
     is_ttest = is_ttest,
@@ -312,6 +382,7 @@
     is_onewaytest = is_oneway,
     is_chi2test = is_chi2test,
     is_ranktest = is_ranktest,
+    is_levenetest = is_levenetest,
     is_xtab = is_xtab,
     is_proptest = is_proptest,
     is_binomtest = is_binomtest,
@@ -324,8 +395,7 @@
 }
 
 .get_ordinal_link <- function(x) {
-  switch(
-    x$link,
+  switch(x$link,
     logistic = "logit",
     cloglog = "log",
     x$link
@@ -333,7 +403,6 @@
 }
 
 
-#' @importFrom stats gaussian binomial Gamma
 .make_tobit_family <- function(x, dist = NULL) {
   if (is.null(dist)) {
     if (inherits(x, "flexsurvreg")) {
@@ -342,8 +411,7 @@
       dist <- x$dist
     }
   }
-  f <- switch(
-    dist,
+  f <- switch(dist,
     gaussian = stats::gaussian("identity"),
     logistic = stats::binomial("logit"),
     llogis = ,
@@ -366,9 +434,8 @@
 
 
 .classify_BFBayesFactor <- function(x) {
-  if (!requireNamespace("BayesFactor", quietly = TRUE)) {
-    stop("This function needs `BayesFactor` to be installed.")
-  }
+  # installed?
+  check_if_installed("BayesFactor")
 
   if (any(class(x@denominator) %in% c("BFcorrelation"))) {
     "correlation"
