@@ -1,6 +1,6 @@
-if (require("testthat") &&
-  require("insight") &&
-  require("stats")) {
+if (requiet("testthat") &&
+  requiet("insight") &&
+  requiet("stats")) {
   data(npk)
   m1 <- aov(yield ~ N * P * K + Error(block), data = npk)
   m2 <- aov(yield ~ N * P * K, data = npk)
@@ -57,19 +57,22 @@ if (require("testthat") &&
     expect_length(find_formula(m1), 1)
     expect_equal(
       find_formula(m1),
-      list(conditional = as.formula("yield ~ N * P * K + Error(block)"))
+      list(conditional = as.formula("yield ~ N * P * K + Error(block)")),
+      ignore_attr = TRUE
     )
     expect_length(find_formula(m2), 1)
     expect_equal(
       find_formula(m2),
-      list(conditional = as.formula("yield ~ N * P * K"))
+      list(conditional = as.formula("yield ~ N * P * K")),
+      ignore_attr = TRUE
     )
   })
 
   test_that("find_terms", {
     expect_equal(find_terms(m1), list(
       response = "yield",
-      conditional = c("N", "P", "K", "Error(block)")
+      conditional = c("N", "P", "K"),
+      error = "Error(block)"
     ))
     expect_equal(
       find_terms(m1, flatten = TRUE),
