@@ -1,8 +1,8 @@
 .runThisTest <- Sys.getenv("RunAllinsightTests") == "yes"
 
-if (.runThisTest || Sys.getenv("USER") == "travis") {
-  if (require("testthat") &&
-    require("insight") && require("glmmTMB") && require("biglm")) {
+if (.runThisTest) {
+  if (requiet("testthat") &&
+    requiet("insight") && requiet("glmmTMB") && requiet("biglm")) {
     data(Salamanders)
     Salamanders$cover <- abs(Salamanders$cover)
 
