@@ -1,20 +1,50 @@
 .runThisTest <- Sys.getenv("RunAllinsightTests") == "yes"
 
 if (.runThisTest) {
-  if (suppressWarnings(require("testthat") &&
-    require("insight") &&
-    require("brms"))) {
+  if (suppressWarnings(requiet("testthat") &&
+    requiet("insight") &&
+    requiet("brms"))) {
 
     # Model fitting -----------------------------------------------------------
 
-    m1 <- insight::download_model("brms_mixed_6")
+    m1 <- suppressWarnings(insight::download_model("brms_mixed_6"))
     m2 <- insight::download_model("brms_mv_4")
     m3 <- insight::download_model("brms_2")
     m4 <- insight::download_model("brms_zi_3")
     m5 <- insight::download_model("brms_mv_5")
     m6 <- insight::download_model("brms_corr_re1")
+    m7 <- suppressWarnings(insight::download_model("brms_mixed_8"))
+    m8 <- insight::download_model("brms_ordinal_1")
 
     # Tests -------------------------------------------------------------------
+    test_that("get_predicted.brmsfit: ordinal dv", {
+      skip_if_not_installed("bayestestR")
+
+      pred1 <- get_predicted(m8)
+      pred2 <- get_predicted(m8, ci_method = "hdi")
+      expect_true(inherits(pred1, "get_predicted"))
+      expect_true(inherits(pred1, "data.frame"))
+      expect_true(all(c("Row", "Response") %in% colnames(pred1)))
+
+      # ci_method changes intervals but not se or predicted
+      pred1 <- data.frame(pred1)
+      pred2 <- data.frame(pred2)
+      expect_equal(pred1$Row, pred2$Row)
+      expect_equal(pred1$Response, pred2$Response)
+      expect_equal(pred1$Predicted, pred2$Predicted)
+      expect_equal(pred1$SE, pred2$SE)
+      expect_false(mean(pred1$CI_low == pred2$CI_low) > 0.1) # most CI bounds are different
+      expect_false(mean(pred1$CI_high == pred2$CI_high) > 0.1) # most CI bounds are different
+
+      # compare to manual predictions
+      pred3 <- get_predicted(m8, centrality_function = stats::median)
+      manual <- rstantools::posterior_epred(m8)
+      manual <- apply(manual[, , 1], 2, median)
+      expect_equal(pred3$Predicted[1:32], manual)
+      manual <- rstantools::posterior_epred(m8)
+      manual <- apply(manual[, , 1], 2, mean)
+      expect_equal(pred1$Predicted[1:32], manual)
+    })
 
     test_that("find_statistic", {
       expect_null(find_statistic(m1))
@@ -24,6 +54,11 @@ if (.runThisTest) {
       expect_null(find_statistic(m5))
     })
 
+    test_that("n_parameters", {
+      expect_equal(n_parameters(m1), 65)
+      expect_equal(n_parameters(m1, effects = "fixed"), 5)
+    })
+
     test_that("model_info", {
       expect_true(model_info(m3)$is_trial)
       expect_true(model_info(m5)[[1]]$is_zero_inflated)
@@ -176,7 +211,9 @@ if (.runThisTest) {
         list(
           response = "y",
           conditional = "x",
-          random = "id"
+          random = "id",
+          sigma = "x",
+          sigma_random = "id"
         )
       )
       expect_identical(
@@ -277,9 +314,9 @@ if (.runThisTest) {
             "b_Age",
             "b_Base",
             "b_Trt1",
-            "b_Base.Trt1"
+            "b_Base:Trt1"
           ),
-          random = c(sprintf("r_patient.%i.Intercept.", 1:59), "sd_patient__Intercept")
+          random = c(sprintf("r_patient[%i,Intercept]", 1:59), "sd_patient__Intercept")
         )
       )
 
@@ -313,20 +350,20 @@ if (.runThisTest) {
         find_parameters(m4),
         list(
           conditional = c("b_Intercept", "b_child", "b_camper"),
-          random = c(sprintf("r_persons.%i.Intercept.", 1:4), "sd_persons__Intercept"),
+          random = c(sprintf("r_persons[%i,Intercept]", 1:4), "sd_persons__Intercept"),
           zero_inflated = c("b_zi_Intercept", "b_zi_child", "b_zi_camper"),
-          zero_inflated_random = c(sprintf("r_persons__zi.%i.Intercept.", 1:4), "sd_persons__zi_Intercept")
+          zero_inflated_random = c(sprintf("r_persons__zi[%i,Intercept]", 1:4), "sd_persons__zi_Intercept")
         )
       )
 
       expect_equal(
-        find_parameters(m5),
+        find_parameters(m5, effects = "all"),
         structure(list(
           count = list(
             conditional = c("b_count_Intercept", "b_count_child", "b_count_camper"),
-            random = c(sprintf("r_persons__count.%i.Intercept.", 1:4), "sd_persons__count_Intercept"),
+            random = c(sprintf("r_persons__count[%i,Intercept]", 1:4), "sd_persons__count_Intercept"),
             zero_inflated = c("b_zi_count_Intercept", "b_zi_count_camper"),
-            zero_inflated_random = c(sprintf("r_persons__zi_count.%i.Intercept.", 1:4), "sd_persons__zi_count_Intercept")
+            zero_inflated_random = c(sprintf("r_persons__zi_count[%i,Intercept]", 1:4), "sd_persons__zi_count_Intercept")
           ),
           count2 = list(
             conditional = c(
@@ -334,9 +371,9 @@ if (.runThisTest) {
               "b_count2_child",
               "b_count2_livebait"
             ),
-            random = c(sprintf("r_persons__count2.%i.Intercept.", 1:4), "sd_persons__count2_Intercept"),
+            random = c(sprintf("r_persons__count2[%i,Intercept]", 1:4), "sd_persons__count2_Intercept"),
             zero_inflated = c("b_zi_count2_Intercept", "b_zi_count2_child"),
-            zero_inflated_random = c(sprintf("r_persons__zi_count2.%i.Intercept.", 1:4), "sd_persons__zi_count2_Intercept")
+            zero_inflated_random = c(sprintf("r_persons__zi_count2[%i,Intercept]", 1:4), "sd_persons__zi_count2_Intercept")
           )
         ),
         "is_mv" = "1"
@@ -363,51 +400,46 @@ if (.runThisTest) {
       expect_equal(
         colnames(get_parameters(m4, effects = "all")),
         c(
-          "b_Intercept", "b_child", "b_camper", "r_persons.1.Intercept.",
-          "r_persons.2.Intercept.", "r_persons.3.Intercept.", "r_persons.4.Intercept.",
+          "b_Intercept", "b_child", "b_camper", "r_persons[1,Intercept]",
+          "r_persons[2,Intercept]", "r_persons[3,Intercept]", "r_persons[4,Intercept]",
           "sd_persons__Intercept", "b_zi_Intercept", "b_zi_child", "b_zi_camper",
-          "r_persons__zi.1.Intercept.", "r_persons__zi.2.Intercept.", "r_persons__zi.3.Intercept.",
-          "r_persons__zi.4.Intercept.", "sd_persons__zi_Intercept"
+          "r_persons__zi[1,Intercept]", "r_persons__zi[2,Intercept]", "r_persons__zi[3,Intercept]",
+          "r_persons__zi[4,Intercept]", "sd_persons__zi_Intercept"
         )
       )
       expect_equal(
-        colnames(get_parameters(m4, effects = "random", component = "cond")),
+        colnames(get_parameters(m4, effects = "random", component = "conditional")),
         c(
-          "r_persons.1.Intercept.", "r_persons.2.Intercept.", "r_persons.3.Intercept.",
-          "r_persons.4.Intercept.", "sd_persons__Intercept"
+          "r_persons[1,Intercept]", "r_persons[2,Intercept]", "r_persons[3,Intercept]",
+          "r_persons[4,Intercept]", "sd_persons__Intercept"
         )
       )
       expect_equal(
-        colnames(get_parameters(m5, effects = "random", component = "cond")),
+        colnames(get_parameters(m5, effects = "random", component = "conditional")),
         c(
-          "r_persons__count.1.Intercept.",
-          "r_persons__count.2.Intercept.",
-          "r_persons__count.3.Intercept.",
-          "r_persons__count.4.Intercept.",
-          "sd_persons__count_Intercept",
-          "r_persons__count2.1.Intercept.",
-          "r_persons__count2.2.Intercept.",
-          "r_persons__count2.3.Intercept.",
-          "r_persons__count2.4.Intercept.",
-          "sd_persons__count2_Intercept"
+          "r_persons__count[1,Intercept]", "r_persons__count[2,Intercept]",
+          "r_persons__count[3,Intercept]", "r_persons__count[4,Intercept]",
+          "sd_persons__count_Intercept", "r_persons__count2[1,Intercept]",
+          "r_persons__count2[2,Intercept]", "r_persons__count2[3,Intercept]",
+          "r_persons__count2[4,Intercept]", "sd_persons__count2_Intercept"
         )
       )
 
       expect_equal(
         colnames(get_parameters(m5, effects = "all", component = "all")),
         c(
-          "b_count_Intercept", "b_count_child", "b_count_camper", "r_persons__count.1.Intercept.",
-          "r_persons__count.2.Intercept.", "r_persons__count.3.Intercept.",
-          "r_persons__count.4.Intercept.", "sd_persons__count_Intercept",
-          "b_zi_count_Intercept", "b_zi_count_camper", "r_persons__zi_count.1.Intercept.",
-          "r_persons__zi_count.2.Intercept.", "r_persons__zi_count.3.Intercept.",
-          "r_persons__zi_count.4.Intercept.", "sd_persons__zi_count_Intercept",
+          "b_count_Intercept", "b_count_child", "b_count_camper", "r_persons__count[1,Intercept]",
+          "r_persons__count[2,Intercept]", "r_persons__count[3,Intercept]",
+          "r_persons__count[4,Intercept]", "sd_persons__count_Intercept",
+          "b_zi_count_Intercept", "b_zi_count_camper", "r_persons__zi_count[1,Intercept]",
+          "r_persons__zi_count[2,Intercept]", "r_persons__zi_count[3,Intercept]",
+          "r_persons__zi_count[4,Intercept]", "sd_persons__zi_count_Intercept",
           "b_count2_Intercept", "b_count2_child", "b_count2_livebait",
-          "r_persons__count2.1.Intercept.", "r_persons__count2.2.Intercept.",
-          "r_persons__count2.3.Intercept.", "r_persons__count2.4.Intercept.",
+          "r_persons__count2[1,Intercept]", "r_persons__count2[2,Intercept]",
+          "r_persons__count2[3,Intercept]", "r_persons__count2[4,Intercept]",
           "sd_persons__count2_Intercept", "b_zi_count2_Intercept", "b_zi_count2_child",
-          "r_persons__zi_count2.1.Intercept.", "r_persons__zi_count2.2.Intercept.",
-          "r_persons__zi_count2.3.Intercept.", "r_persons__zi_count2.4.Intercept.",
+          "r_persons__zi_count2[1,Intercept]", "r_persons__zi_count2[2,Intercept]",
+          "r_persons__zi_count2[3,Intercept]", "r_persons__zi_count2[4,Intercept]",
           "sd_persons__zi_count2_Intercept"
         )
       )
@@ -468,25 +500,35 @@ if (.runThisTest) {
 
     test_that("get_priors", {
       expect_equal(
-        get_priors(m1),
+        get_priors(m7),
         data.frame(
-          Parameter = c("Age", "Base", "Base:Trt1", "Trt1"),
-          Distribution = c("student_t", "student_t", "student_t", "student_t"),
-          Location = c("5, 0", "5, 0", "5, 0", "5, 0"),
-          Scale = c(10, 10, 10, 10),
+          Parameter = c(
+            "b_Intercept", "b_Age", "b_Base", "b_Trt1", "b_Base:Trt1",
+            "sd_patient__Intercept", "sd_patient__Age",
+            "cor_patient__Intercept__Age"
+          ),
+          Distribution = c(
+            "student_t", "student_t", "student_t",
+            "student_t", "student_t", "cauchy", "cauchy", "lkj"
+          ),
+          Location = c(1.4, 0, 0, 0, 0, NA, NA, 1),
+          Scale = c(2.5, 10, 10, 10, 10, NA, NA, NA),
+          df = c(3, 5, 5, 5, 5, NA, NA, NA),
           stringsAsFactors = FALSE
-        )
+        ),
+        ignore_attr = TRUE
       )
-
       expect_equal(
         get_priors(m3),
         data.frame(
-          Parameter = c("c2", "treat1", "treat1:c2"),
-          Distribution = c("uniform", "uniform", "uniform"),
-          Location = c(0, 0, 0),
-          Scale = c(NA, NA, NA),
+          Parameter = c("b_Intercept", "b_treat1", "b_c2", "b_treat1:c2"),
+          Distribution = c("student_t", "uniform", "uniform", "uniform"),
+          Location = c(0, NA, NA, NA),
+          Scale = c(2.5, NA, NA, NA),
+          df = c(3, NA, NA, NA),
           stringsAsFactors = FALSE
-        )
+        ),
+        ignore_attr = TRUE
       )
     })
 
@@ -500,18 +542,18 @@ if (.runThisTest) {
               "b_Intercept",
               "b_child",
               "b_camper",
-              "r_persons.1.Intercept.",
-              "r_persons.2.Intercept.",
-              "r_persons.3.Intercept.",
-              "r_persons.4.Intercept.",
+              "r_persons[1,Intercept]",
+              "r_persons[2,Intercept]",
+              "r_persons[3,Intercept]",
+              "r_persons[4,Intercept]",
               "sd_persons__Intercept",
               "b_zi_Intercept",
               "b_zi_child",
               "b_zi_camper",
-              "r_persons__zi.1.Intercept.",
-              "r_persons__zi.2.Intercept.",
-              "r_persons__zi.3.Intercept.",
-              "r_persons__zi.4.Intercept.",
+              "r_persons__zi[1,Intercept]",
+              "r_persons__zi[2,Intercept]",
+              "r_persons__zi[3,Intercept]",
+              "r_persons__zi[4,Intercept]",
               "sd_persons__zi_Intercept"
             ),
             Effects = c(
@@ -607,29 +649,29 @@ if (.runThisTest) {
               "b_count2_Intercept",
               "b_count2_child",
               "b_count2_livebait",
-              "r_persons__count.1.Intercept.",
-              "r_persons__count.2.Intercept.",
-              "r_persons__count.3.Intercept.",
-              "r_persons__count.4.Intercept.",
+              "r_persons__count[1,Intercept]",
+              "r_persons__count[2,Intercept]",
+              "r_persons__count[3,Intercept]",
+              "r_persons__count[4,Intercept]",
               "sd_persons__count_Intercept",
-              "r_persons__count2.1.Intercept.",
-              "r_persons__count2.2.Intercept.",
-              "r_persons__count2.3.Intercept.",
-              "r_persons__count2.4.Intercept.",
+              "r_persons__count2[1,Intercept]",
+              "r_persons__count2[2,Intercept]",
+              "r_persons__count2[3,Intercept]",
+              "r_persons__count2[4,Intercept]",
               "sd_persons__count2_Intercept",
               "b_zi_count_Intercept",
               "b_zi_count_camper",
               "b_zi_count2_Intercept",
               "b_zi_count2_child",
-              "r_persons__zi_count.1.Intercept.",
-              "r_persons__zi_count.2.Intercept.",
-              "r_persons__zi_count.3.Intercept.",
-              "r_persons__zi_count.4.Intercept.",
+              "r_persons__zi_count[1,Intercept]",
+              "r_persons__zi_count[2,Intercept]",
+              "r_persons__zi_count[3,Intercept]",
+              "r_persons__zi_count[4,Intercept]",
               "sd_persons__zi_count_Intercept",
-              "r_persons__zi_count2.1.Intercept.",
-              "r_persons__zi_count2.2.Intercept.",
-              "r_persons__zi_count2.3.Intercept.",
-              "r_persons__zi_count2.4.Intercept.",
+              "r_persons__zi_count2[1,Intercept]",
+              "r_persons__zi_count2[2,Intercept]",
+              "r_persons__zi_count2[3,Intercept]",
+              "r_persons__zi_count2[4,Intercept]",
               "sd_persons__zi_count2_Intercept"
             ),
             Effects = c(
