@@ -1,9 +1,11 @@
 .runThisTest <- Sys.getenv("RunAllinsightTests") == "yes"
 
 if (.runThisTest) {
-  if (require("testthat") && require("insight") && require("mgcv")) {
+  if (requiet("testthat") && requiet("insight") && requiet("mgcv")) {
     set.seed(123)
-    dat <- gamSim(1, n = 400, dist = "normal", scale = 2)
+    void <- capture.output(
+      dat <- mgcv::gamSim(1, n = 400, dist = "normal", scale = 2)
+    )
     m1 <- mgcv::gam(y ~ s(x0) + s(x1) + s(x2) + s(x3), data = dat)
 
     m2 <- download_model("gam_zi_1")
@@ -15,6 +17,11 @@ if (.runThisTest) {
       expect_true(model_info(m3)$is_multivariate)
     })
 
+    test_that("n_parameters", {
+      expect_equal(n_parameters(m1), 5)
+      expect_equal(n_parameters(m1, component = "conditional"), 1)
+    })
+
     test_that("clean_names", {
       expect_equal(clean_names(m1), c("y", "x0", "x1", "x2", "x3"))
       expect_equal(clean_names(m2), c("y", "x2", "x3", "x0", "x1"))
@@ -44,6 +51,14 @@ if (.runThisTest) {
       expect_identical(find_response(m3), c(y0 = "y0", y1 = "y1"))
     })
 
+    test_that("find_smooth", {
+      expect_identical(find_smooth(m1), list(smooth_terms = c("s(x0)", "s(x1)", "s(x2)", "s(x3)")))
+    })
+
+    test_that("get_call", {
+      expect_identical(deparse(get_call(m1)), "mgcv::gam(formula = y ~ s(x0) + s(x1) + s(x2) + s(x3), data = dat)")
+    })
+
     test_that("get_response", {
       expect_equal(get_response(m1), dat$y)
       expect_equal(length(get_response(m2)), 500)
@@ -178,5 +193,36 @@ if (.runThisTest) {
     test_that("find_statistic", {
       expect_identical(find_statistic(m1), "t-statistic")
     })
+
+    test_that("get_predicted", {
+      tmp <- mgcv::gam(y ~ s(x0) + s(x1), data = head(dat, 30))
+      pred <- get_predicted(tmp)
+      expect_s3_class(pred, "get_predicted")
+      expect_equal(
+        as.vector(pred),
+        c(
+          11.99341, 5.58098, 10.89252, 7.10335, 5.94836, 6.5724, 8.5054,
+          5.47147, 5.9343, 8.27001, 5.71199, 9.94999, 5.69979, 6.63532,
+          6.00475, 5.58633, 11.54848, 6.1083, 6.6151, 5.37164, 6.86236,
+          7.80726, 7.38088, 5.70664, 10.60654, 7.62847, 5.8596, 6.06744,
+          5.81571, 10.4606
+        ),
+        tolerance = 1e-3
+      )
+
+      x <- get_predicted(tmp, predict = NULL, type = "link")
+      y <- get_predicted(tmp, predict = "link")
+      z <- predict(tmp, type = "link", se.fit = TRUE)
+      expect_equal(x, y)
+      expect_equal(x, z$fit, ignore_attr = TRUE)
+      expect_equal(as.data.frame(x)$SE, z$se.fit, ignore_attr = TRUE)
+
+      x <- get_predicted(tmp, predict = NULL, type = "response")
+      y <- get_predicted(tmp, predict = "expectation")
+      z <- predict(tmp, type = "response", se.fit = TRUE)
+      expect_equal(x, y, ignore_attr = TRUE)
+      expect_equal(x, z$fit, ignore_attr = TRUE)
+      expect_equal(as.data.frame(x)$SE, z$se.fit, ignore_attr = TRUE)
+    })
   }
 }
