@@ -2,7 +2,7 @@
 
 if (.runThisTest) {
   unloadNamespace("gam")
-  if (require("testthat") && require("insight") && require("mgcv")) {
+  if (requiet("testthat") && requiet("insight") && requiet("mgcv")) {
     set.seed(0)
     dat <- gamSim(6, n = 200, scale = .2, dist = "poisson")
     m1 <-
@@ -10,7 +10,8 @@ if (.runThisTest) {
         y ~ s(x0) + s(x1) + s(x2),
         family = poisson,
         data = dat,
-        random = list(fac = ~1)
+        random = list(fac = ~1),
+        verbosePQL = FALSE
       )
 
     test_that("model_info", {
@@ -114,14 +115,16 @@ if (.runThisTest) {
     ra <- rep(rnorm(10), rep(4 * n.g, 10))
     fb <- as.factor(rep(rep(1:4, rep(n.g, 4)), 10))
     rb <- rep(rnorm(4), rep(n.g, 4))
-    for (i in 1:9)
+    for (i in 1:9) {
       rb <- c(rb, rep(rnorm(4), rep(n.g, 4)))
+    }
     ## simulate auto-correlated errors within groups
     e <- array(0, 0)
     for (i in 1:40) {
       eg <- rnorm(n.g, 0, sig)
-      for (j in 2:n.g)
+      for (j in 2:n.g) {
         eg[j] <- eg[j - 1] * 0.6 + eg[j]
+      }
       e <- c(e, eg)
     }
     dat$y <- f + ra + rb + e
@@ -132,17 +135,21 @@ if (.runThisTest) {
     m1 <- gamm(
       y ~ s(x0, bs = "cr") + s(x1, bs = "cr"),
       data = dat,
-      random = list(fa =  ~ 1, fb =  ~ 1),
+      random = list(fa = ~1, fb = ~1),
       correlation = corAR1()
     )
 
     set.seed(0)
-    dat <- gamSim(6, n = 200, scale = .2, dist = "poisson")
+
+    void <- capture.output(
+      dat <- gamSim(6, n = 200, scale = .2, dist = "poisson")
+    )
 
     m2 <- gamm(
       y ~ s(x0) + s(x1) + s(x2),
       family = poisson,
-      data = dat
+      data = dat,
+      verbosePQL = FALSE
     )
 
     dat$g <- dat$fac
@@ -150,14 +157,17 @@ if (.runThisTest) {
       y ~ s(x0) + s(x1) + s(x2),
       family = poisson,
       data = dat,
-      random = list(g = ~1)
+      random = list(g = ~1),
+      verbosePQL = FALSE
     )
 
     test_that("find_formula-gamm-1", {
       expect_equal(
         find_formula(m1),
-        list(conditional = as.formula("y ~ s(x0, bs = \"cr\") + s(x1, bs = \"cr\")"),
-             random = list(as.formula("~1 | fa"), as.formula("~1 | fb"))),
+        list(
+          conditional = as.formula("y ~ s(x0, bs = \"cr\") + s(x1, bs = \"cr\")"),
+          random = list(as.formula("~1 | fa"), as.formula("~1 | fb"))
+        ),
         ignore_attr = TRUE
       )
     })
@@ -173,11 +183,12 @@ if (.runThisTest) {
     test_that("find_formula-gamm-3", {
       expect_equal(
         find_formula(m3),
-        list(conditional = as.formula("y ~ s(x0) + s(x1) + s(x2)"),
-             random = as.formula("~1 | g")),
+        list(
+          conditional = as.formula("y ~ s(x0) + s(x1) + s(x2)"),
+          random = as.formula("~1 | g")
+        ),
         ignore_attr = TRUE
       )
     })
-
   }
 }
