@@ -1,6 +1,6 @@
-if (require("testthat") &&
-  require("insight") &&
-  require("logistf")) {
+if (requiet("testthat") &&
+  requiet("insight") &&
+  requiet("logistf")) {
   data(sex2)
   m1 <- logistf(case ~ age + oc + vic + vicl + vis + dia, data = sex2)
 
