@@ -1,4 +1,6 @@
-if (require("testthat") && require("insight") && require("metaBMA")) {
+.runThisTest <- Sys.getenv("RunAllinsightTests") == "yes"
+
+if (.runThisTest && requiet("testthat") && requiet("insight") && requiet("metaBMA")) {
   data(towels)
   set.seed(123)
   mf <- meta_fixed(logOR, SE, study, data = towels, d = prior("norm", c(mean = 0, sd = .3), lower = 0))
@@ -11,9 +13,11 @@ if (require("testthat") && require("insight") && require("metaBMA")) {
 
 
   set.seed(123)
-  mr <- meta_random(logOR, SE, study, data = towels,
-                    d = prior("cauchy", c(location = 0, scale = 0.707)),
-                    tau = prior("invgamma", c(shape = 1, scale = 0.15)))
+  mr <- meta_random(logOR, SE, study,
+    data = towels,
+    d = prior("cauchy", c(location = 0, scale = 0.707)),
+    tau = prior("invgamma", c(shape = 1, scale = 0.15))
+  )
 
   test_that("get_priors-metaBMA", {
     priors <- get_priors(mr)
